/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core;

import modesteditor.core.model.IModestSimulationListener;
import modesteditor.core.model.Model;
import modesteditor.core.model.ModestAction;
import modesteditor.core.model.ModestSimulation;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;

/**
 * The page that is shown in the Outline view. It's only responsible for
 * managing the internal tree viewer, which is what actually displays the
 * tree of the program's bindings. We pass the tree viewer only the ModestSimulation
 * as input, and it finds out the tree structure using the
 * ModestContentProvider and ModestLabelProvider.
 * <p>
 * See http://www.eclipse.org/articles/treeviewer-cg/TreeViewerArticle.htm
 * for a good explanation of how this works.
 * */
public class ModestContentOutlinePage extends ContentOutlinePage implements IModestSimulationListener {
	
	
	/** The editor whose outline is to be displayed. */
	private ModestEditor editor;
	
	/** Creates a new outline page for the given editor. */
	public ModestContentOutlinePage(ModestEditor editor) {
		super();
		this.editor = editor;
		editor.getSimulation().addListener(this);
	}
	
	/** Sets up the internal tree viewer to use MoDeST's content and label
     * providers, to inform this page when the user selects a model, and
     * to display the models in the editor's program. */
    public void createControl (Composite parent) {
        super.createControl(parent);
        TreeViewer viewer = getTreeViewer();
        viewer.setAutoExpandLevel(2);
        viewer.setContentProvider(new ModestContentProvider());
        viewer.setLabelProvider(new ModestLabelProvider());
        viewer.addSelectionChangedListener(this);
        viewer.setInput(editor.getSimulation());
    }
    
    /** Refreshes the tree viewer when the simulation is updated. */
    private void update () {
		TreeViewer viewer = getTreeViewer();
		if (viewer != null) {
			Control control = viewer.getControl();
			if (control != null && !control.isDisposed()) {
		    	viewer.removeSelectionChangedListener(this);
				control.setRedraw(false);
				viewer.refresh();
				viewer.expandToLevel(2);
				control.setRedraw(true);
		    	viewer.addSelectionChangedListener(this);
			}
		}
    }
	
    /** Called when the program is updated. Updates the outline.*/
    /* This has to be done by passing a runnable to Eclipse's UI thread,
     * because other threads aren't allowed to mess with the UI. */
    public void simulationChanged (ModestSimulation sim) {
    	if (!getControl().isDisposed()) {
    		getControl().getDisplay().asyncExec(new Runnable () {
				public void run () {
					update();
				}
    		});
    	}
    }
    
    /** When the user selects a model in the tree, selects the 
     * corresponding identifier in the editor. */
    // TODO check if the offset is correct for Models
	public void selectionChanged (SelectionChangedEvent event) {
		super.selectionChanged(event);
		if (event.getSelection() instanceof IStructuredSelection) {
			IStructuredSelection selection = (IStructuredSelection) event.getSelection();
			if (selection.size() != 1) return;
			Model sel = ((Model) selection.getFirstElement());
			
			try {
				IDocument document = editor.getDocumentProvider().getDocument(editor.getEditorInput());
				int lineOffset = document.getLineOffset(sel.getLine()-1);
				//System.err.println("Line Offset of selected: " + lineOffset);
				int globalOffset = lineOffset + sel.getColumn()-1;
				//System.err.println("Global Offset of selected: " + globalOffset);
				editor.selectAndReveal(globalOffset, sel.getLength());
			} catch (BadLocationException e) {
				System.err.println("Bad line offset for the selected outline element.");
			}
			
		}
	}
    
	/** Selects the given model in the tree.
	 * Prevents selection change events so that selectionChanged() doesn't
	 * get called, which would create an infinite loop of events. */ 
    public void select (Model sel) {
    	TreeViewer viewer = getTreeViewer();
    	viewer.removeSelectionChangedListener(this);
    	viewer.setSelection(new StructuredSelection(sel), true);
    	viewer.addSelectionChangedListener(this);
    }
    
    /** Deselects everything selected in the tree. */
    public void deselect () {
    	getTreeViewer().setSelection(new StructuredSelection());
    }
    
    
    /* ****************************************************************************************
     *	
     * Old Code that didn't work
     *
     *****************************************************************************************/
    
	/*private TreeViewer viewer;
	
	//In dieser Klasse passiert das parsen, dann kann man das root elem dem ContentProvider geben.
	
	protected Object fInput;
	protected IDocumentProvider fDocumentProvider;
	protected ITextEditor fTextEditor;
	
	protected Process viewerRoot;
	
	*//**
	 * Creates a content outline page using the given provider and the given editor.
	 * 
	 * @param provider the document provider
	 * @param editor the editor
	 *//*
	public ModestContentOutlinePage(IDocumentProvider provider, ITextEditor editor) {
		super();
		fDocumentProvider = provider;
		fTextEditor = editor;
	}
	
	 (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 
	public void createControl(Composite parent) {

		super.createControl(parent);

		viewer= new TreeViewer(parent);
		viewer.setContentProvider(new ModestContentProvider(fInput, fDocumentProvider));
		viewer.setLabelProvider(new ModestLabelProvider());
		viewer.addSelectionChangedListener(this);
		((ModestContentProvider)viewer.getContentProvider()).setContentOutlinePage(this);

		//TODO make a Process out of the input.
		if (fInput != null && viewerRoot == null) {
			String doc = fDocumentProvider.getDocument(fInput).get();
			if (doc != null) {
				viewerRoot = parse(doc);
				viewer.setInput(viewerRoot);
				viewer.expandAll();
				System.err.println("Viewers input: " + viewer.getInput().toString());
				System.err.println("Viewers tree itemcount: " + viewer.getTree().getItemCount());
				System.err.println("Viewers tree items: " + viewer.getTree().getItems().toString());
			}
			
		} else viewer.setInput(viewerRoot);
		viewer.expandAll();
	}
	
	 (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 
	public void selectionChanged(SelectionChangedEvent event) {

		super.selectionChanged(event);

		ISelection selection= event.getSelection();
		if (selection.isEmpty())
			fTextEditor.resetHighlightRange();
		else {
			Model mod = (Model) ((IStructuredSelection) selection).getFirstElement();
			int start = mod.getOffset();
			int length = mod.getLength();
			try {
				fTextEditor.setHighlightRange(start, length, true);
			} catch (IllegalArgumentException x) {
				fTextEditor.resetHighlightRange();
			}
		}
	}
	
	*//**
	 * Sets the input of the outline page
	 * 
	 * @param input the input of this outline page
	 *//*
	public void setInput(Object input) {
		fInput= input;
		update();
	}
	
	*//**
	 * Updates the outline page.
	 *//*
	public void update() {
		TreeViewer viewer= getTreeViewer();
		try {
			if (viewer != null && fInput != null) {
				Control control= viewer.getControl();
				if (control != null && !control.isDisposed()) {
					control.setRedraw(false);
					//viewer.setInput(fInput);
					String doc = fDocumentProvider.getDocument(fInput).get();
					if (doc != null) {
						viewerRoot = parse(doc);
						viewer.setInput(viewerRoot);
						viewer.expandAll();
						System.err.println("Viewers input: " + viewer.getInput().toString());
						System.err.println("Viewers tree itemcount: " + viewer.getTree().getItemCount());
						System.err.println("Viewers tree items: " + viewer.getTree().getItems().toString());
						
					}
					control.setRedraw(true);
				}
			}
		} catch (Exception e) {
			System.err.println("Exception in contentOutlinePage.update(): " + e);
		}
	}
	
	public void setViewerRoot(Process root) {
		viewerRoot = root;
	}
	
	public TreeViewer getTreeViewer() {
		return viewer;
	}
	
	protected Process parse(String input) {
		
		//do the parsing here
		
		try {
		      StringReader reader = new StringReader(input);

		      ModestLexer lexer = new ModestLexer(reader); 

		      ModestParser parser = new ModestParser(lexer);
		      parser.simulation();

		      CommonAST parseTree = (CommonAST)parser.getAST();
		      System.err.println(parseTree.toStringList());
		      

		      ModestTreeWalker walker = new ModestTreeWalker();
		      Process root =  (Process)walker.simulation(parseTree);
		      System.err.println("Root nach dem parsen: "+ root.toString());
		      return root;
		      
		    } catch(Exception e) { System.err.println("Exception in parse: "+e); }
		    return null;
	}*/
	
}
