package modesteditor.core;

import modesteditor.core.modest.ModestCodeScanner;
import modesteditor.core.util.ModestColorProvider;
import modesteditor.core.util.ModestTextStyleProvider;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle of the complete Modesteditor plugin
 */
public class ModestEditorPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "modesteditor.core";
	public final static String MODEST_PARTITIONING= "__modest_partitioning";

	// The shared instance
	private static ModestEditorPlugin plugin;

	private ModestColorProvider fColorProvider;
	private ModestTextStyleProvider fTextStyleProvider;
	private ModestCodeScanner fCodeScanner;
	private ModestPartitionScanner fPartitionScanner;
	
	/** Color attribute for the color in syntax highlighting. */
	public static final String MULTI_LINE_COMMENT= "modestMultiLineCommentColor";
	/** Color attribute for the color in syntax highlighting. */
	public static final String SINGLE_LINE_COMMENT= "modestSingleLineCommentColor";
	/** Color attribute for the color in syntax highlighting. */
	public static final String KEYWORD= "modestKeywordColor";
	/** Color attribute for the color in syntax highlighting. */
	public static final String TYPE= "modestTypeColor";
	/** Color attribute for the color in syntax highlighting. */
	public static final String DIST= "modestDistColor";
	
	/** Locations of the compiler. */
	//public static final String COMPILER_LOCATION = "modestCompilerLocataion";
	/** Locations of the output folder. */
	//public static final String OUTPUT_FOLDER_LOCATION = "modestOutputFolderLocation";
	
	/** Boolean attribute for whether to mark syntax errors in the editor. */
	public static final String MODEST_MARK_ERRORS = "modestMarkErrors";
	
	/**
	 * The constructor, initializes the plugin variable to fulfill the singleton pattern.
	 */
	public ModestEditorPlugin() {
		plugin = this;
	}

	/** 
	 * Start function which initializes the default preferences of the plugin
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		initializeDefaultPluginPreferences();
	}

	/**
	 * Stop function which releases the context
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance in order to make it a singleton
	 *
	 * @return the shared instance
	 */
	public static ModestEditorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the singleton Modest color provider.
	 * 
	 * @return the singleton Modest color provider
	 */
	public ModestColorProvider getModestColorProvider() {
		if (fColorProvider == null)
			fColorProvider= new ModestColorProvider();
		return fColorProvider;
	}

	/**
	 * Returns the singleton Modest text style provider.
	 * 
	 * @return the singleton Modest text style provider
	 */
	public ModestTextStyleProvider getModestTextStyleProvider() {
		if (fTextStyleProvider == null)
			fTextStyleProvider= new ModestTextStyleProvider();
		return fTextStyleProvider;
	}
	
	/**
	 * Returns the singleton MoDeST code scanner.
	 * 
	 * @return the singleton MoDeST code scanner
	 */
	public RuleBasedScanner getModestCodeScanner() {
		if (fCodeScanner == null)
			fCodeScanner= new ModestCodeScanner(getModestTextStyleProvider());
		return fCodeScanner;
	} 

	/**
	 * Return a scanner for creating MoDeST partitions.
	 * 
	 * @return a scanner for creating MoDeST partitions
	 */
	public ModestPartitionScanner getModestPartitionScanner() {
		if (fPartitionScanner == null)
			fPartitionScanner= new ModestPartitionScanner();
		return fPartitionScanner;
	} 

	/**
	 * Return the image descriptor of the given filename. The images 
	 * have to be located in the icons folder.
	 */	
	public static ImageDescriptor getImageDescriptor(String name) {
		try {
			/*URL installURL = getDefault().getDescriptor().getInstallURL();
			URL url = new URL(installURL, iconPath + name);*/
			/*Bundle bundle = Platform.getBundle("modesteditor.core");
			Path path = new Path("icons/" + name);
			URL url = FileLocator.find(bundle, path, null);
			
			//System.err.println("URL for image: " + url);
			
			return ImageDescriptor.createFromURL(url);*/
			
			
				String url = "/icons/" + name;
				ImageDescriptor descriptor = ImageDescriptor.createFromURL(ModestEditorPlugin.getDefault().getBundle().getEntry(url));
				
			
			return descriptor;
			
			
		/*} catch (MalformedURLException e) {
			// should not happen
			return ImageDescriptor.getMissingImageDescriptor();
		}*/
		} catch (Exception e) {
			System.err.println("Exception in getImageDescriptor for name:" + name);
		}
		return null;
	}
	
	/** Initializes default values for this plug-in's preferences. */
	/* TODO: This is deprecated. See the deprecated comment in
	 * Plugin.initializeDefaultPluginPreferences. It is called 
	 * specifically from the plugin class, not from another plugin class */
	protected void initializeDefaultPluginPreferences () {
		IPreferenceStore store = getPreferenceStore();
		PreferenceConverter.setDefault(store, KEYWORD, new RGB(0,0,128));
		PreferenceConverter.setDefault(store, TYPE, new RGB(0,0,255));
		PreferenceConverter.setDefault(store, DIST, new RGB(128,128,0));
		PreferenceConverter.setDefault(store, MULTI_LINE_COMMENT, new RGB(63,127,95));
		PreferenceConverter.setDefault(store, SINGLE_LINE_COMMENT, new RGB(101,146,191));
		store.setDefault(MODEST_MARK_ERRORS, true);
		savePluginPreferences();
		//System.err.println("Default plugin preferences initialized");
	}
	
	
}
