/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core;

import modesteditor.core.modest.ModestAutoIndentStrategy;
import modesteditor.core.modest.ModestCompletionProcessor;
import modesteditor.core.modest.ModestDoubleClickSelector;
import modesteditor.core.util.ModestTextStyleProvider;

import org.eclipse.jface.text.DefaultIndentLineAutoEditStrategy;
import org.eclipse.jface.text.IAutoEditStrategy;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.reconciler.IReconciler;
import org.eclipse.jface.text.reconciler.MonoReconciler;
import org.eclipse.jface.text.rules.BufferedRuleBasedScanner;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.graphics.RGB;



/**
 * Configuration for an <code>SourceViewer</code> which shows MoDeST code.
 */
public class ModestSourceViewerConfiguration extends SourceViewerConfiguration {

	/**
	 * Single token scanner.
	 */
	static class SingleTokenScanner extends BufferedRuleBasedScanner {
		public SingleTokenScanner(TextAttribute attribute) {
			setDefaultReturnToken(new Token(attribute));
		}
	}
	/** The editor which SourceViewer it configures. */
	private ModestEditor editor;
	
	public ModestSourceViewerConfiguration(ModestEditor editor) {
		super();
		//System.err.println("SourceViewerConfiguration created");
		this.editor = editor;
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public IAnnotationHover getAnnotationHover(ISourceViewer sourceViewer) {
		return new ModestAnnotationHover();
	}
	
	/*
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getAutoEditStrategies(org.eclipse.jface.text.source.ISourceViewer, java.lang.String)
	 */
	public IAutoEditStrategy[] getAutoEditStrategies(ISourceViewer sourceViewer, String contentType) {
		IAutoEditStrategy strategy= (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType) ? new ModestAutoIndentStrategy() : new DefaultIndentLineAutoEditStrategy());
		return new IAutoEditStrategy[] { strategy };
	}
	
	/**
	 * Returns the partitioning for the MoDeST document. 
	 */
	public String getConfiguredDocumentPartitioning(ISourceViewer sourceViewer) {
		return ModestEditorPlugin.MODEST_PARTITIONING;
	}
	
	/**
	 * Returns the content types of the document.
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE /*ModestPartitionScanner.MODEST_DEFAULT*/, ModestPartitionScanner.MODEST_MULTILINE_COMMENT, ModestPartitionScanner.MODEST_SINGLELINE_COMMENT };
	}
	
	/**
	 * Returns the content assistant for a MoDeST document.
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {

		ContentAssistant assistant= new ContentAssistant();
		assistant.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));
		assistant.setContentAssistProcessor(new ModestCompletionProcessor(), IDocument.DEFAULT_CONTENT_TYPE);
		//assistant.setContentAssistProcessor(new JavaDocCompletionProcessor(), JavaPartitionScanner.JAVA_DOC);

		assistant.enableAutoActivation(true);
		assistant.setAutoActivationDelay(500);
		assistant.setProposalPopupOrientation(IContentAssistant.PROPOSAL_OVERLAY);
		assistant.setContextInformationPopupOrientation(IContentAssistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(ModestEditorPlugin.getDefault().getModestColorProvider().getColor(new RGB(150, 150, 0)));

		//System.err.println("Got contentAssistant");
		return assistant;
		
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String getDefaultPrefix(ISourceViewer sourceViewer, String contentType) {
		return (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType) ? "//" : null); 
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public ITextDoubleClickStrategy getDoubleClickStrategy(ISourceViewer sourceViewer, String contentType) {
		return new ModestDoubleClickSelector();
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String[] getIndentPrefixes(ISourceViewer sourceViewer, String contentType) {
		return new String[] { "\t", "    " }; 
	}
	
	/**
	 * Returns the PresentationReconciler. Here it does the syntax highlighting 
	 * specific to a MoDeST file.
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {

		//ModestColorProvider provider= ModestEditorPlugin.getDefault().getModestColorProvider();
		ModestTextStyleProvider styleProvider = ModestEditorPlugin.getDefault().getModestTextStyleProvider();
		
		PresentationReconciler reconciler= new PresentationReconciler();
		reconciler.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));
		
		DefaultDamagerRepairer dr= new DefaultDamagerRepairer(ModestEditorPlugin.getDefault().getModestCodeScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);
				
		dr= new DefaultDamagerRepairer(new SingleTokenScanner(styleProvider.getMultiLineCommentStyle()));
		reconciler.setDamager(dr, ModestPartitionScanner.MODEST_MULTILINE_COMMENT);
		reconciler.setRepairer(dr, ModestPartitionScanner.MODEST_MULTILINE_COMMENT);
		
		dr= new DefaultDamagerRepairer(new SingleTokenScanner(styleProvider.getSingleLineCommentStyle()));
		reconciler.setDamager(dr, ModestPartitionScanner.MODEST_SINGLELINE_COMMENT);
		reconciler.setRepairer(dr, ModestPartitionScanner.MODEST_SINGLELINE_COMMENT);
		
		/*System.err.println("Reconciler partitioning: " + reconciler.getDocumentPartitioning());
		System.err.println("Reconciler: " + reconciler.toString());		
		*/
		
		return reconciler;
	}
	
	/** Returns the reconciler that will update the simulation model
     * in response to changes in the editor's document. */
    public IReconciler getReconciler (ISourceViewer viewer) {
    	MonoReconciler reconciler = new MonoReconciler(new ModestReconcilingStrategy(editor), false);
    	return reconciler;
    }
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return 4;
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public ITextHover getTextHover(ISourceViewer sourceViewer, String contentType) {
		return new ModestTextHover();
	}
}

