/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.model;

import modesteditor.core.antlrparser.ModestLexerNew;
import modesteditor.core.antlrparser.ModestOutlineTreeWalker;
import modesteditor.core.antlrparser.ModestParserNew;
import modesteditor.core.antlrparser.Symbol;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Region;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import antlr.ANTLRException;
import antlr.collections.AST;

/**
 * Maintains the high-level representation of an MoDeST simulation,
 * including the parse tree and the identifiers defined in the program.
 * <p>
 * An ModestSimulation works on one document being edited in the workspace.
 * It has to be updated by the ModestEditor whenever the document is changed.
 *
 */
public class ModestSimulation {

	/** The document containing the program source. */
	private IDocument document;
	/** The list of listeners registered to be notified of updates. */
	private List listeners;
	
	/** The root of the parse tree */
	private Process sim;
	/** The list of the models */
	private List<Model> models;
	
	private ModestLexerNew lexer;
	//private ModestLexer lexer;
	private ModestParserNew parser;
	//private ModestOutlineParser parser;
	
	/** The list of problems. */
	private ArrayList<IRegion> problems;
	/** The list of warnings. */
	private ArrayList<Region> warnings;
	/** The list of variable names, used for completion */
	private ArrayList<String> variables;
	/** The list of process names, used for completion */
	private ArrayList<String> processes;
	
	
	/** The current AST resulted from the last parse run. */
	private AST currentAST;
	
	/** Tells wheter the currentAST is valid or not. */
	private boolean isASTValid;
	
	/** Creates a ModestSimulation attached to the given document.
	 * The ModestSimulation does not update itself when the document changes. */
	public ModestSimulation(IDocument document) {
		this.document = document;
		listeners = new ArrayList();
		problems = new ArrayList<IRegion>();
		warnings = new ArrayList<Region>();
		variables = new ArrayList<String>();
		processes = new ArrayList<String>();
	}
	
	/**
	 * Returns the simulation's root Process which is first 
	 * updated if the document isn't parsed yet.
	 * @return the root of the parse tree.
	 */
	public Process getSim(){
		if (sim == null)
			update();
		return sim;
	}
	
	/** Returns the list of identifiers bound by the program.
	 * See Model for details. */
	public Model[] getModels(){
		if (models == null)
			update();
		return (Model[]) models.toArray(new Model[0]);
	}
	
	/** Returns the regions where problems were found in the simulation.
	 * In this version, only syntax errors are found. */
	public IRegion[] getProblems () {
		//turns out to be an infinit loop in coordination w/ the ModestEditor class
		/*if (problems == null)
			update();*/
		return (IRegion[]) problems.toArray(new IRegion[0]);
	}
	
	/**
	 * Returns the warnings for unused variables.
	 * @return the regions with unused variables.
	 */
	public IRegion[] getWarnings () {
		return (IRegion[]) warnings.toArray(new IRegion[0]);
	}
	
		
	/**
	 * @return the string representation of all variables and exceptions
	 */
	public ArrayList<String> getVariables() {
		return variables;
	}

	
	/**
	 * @return the process names used for completion
	 */
	public ArrayList<String> getProcesses() {
		return processes;
	}

	/** Updates the representation of the program. This method should
	 * be called by the editor when the document is changed. */
	public void update (){
		parse();
		if (sim != null) models = sim.getChildren();
		/*models = sim.getProcesses(); 
		models.addAll(sim.getActions());
		models.addAll(sim.getExceptions());
		models.addAll(sim.getVariables());
		models.addAll(sim.getTypes());
		*/fireModelChanged();
	}
	
	private void addLineProblem (int line){
		try {
			if (problems.isEmpty()){
				int lineOffset = document.getLineOffset(line-1);
				int nextLineOffset = document.getLineOffset(line);
				problems.add(new Region(lineOffset,nextLineOffset-lineOffset-1));
				
			} else {
				boolean ignore = false;
				for (int i = 0; i < problems.size(); i++) {
					IRegion problem = problems.get(i);
					int lineOffset = document.getLineOffset(line-1);
					if (problem.getOffset() <= lineOffset && lineOffset <= (problem.getOffset()+problem.getLength()))
							ignore = true;
				}
				
				if (!ignore){
					int lineOffset = document.getLineOffset(line-1);
					int nextLineOffset = document.getLineOffset(line);
					problems.add(new Region(lineOffset,nextLineOffset-lineOffset-1));
					
				}
				
				/*for (Iterator iter = problems.iterator(); iter.hasNext();) {
					IRegion element = (IRegion) iter.next();
					int lineOffset = document.getLineOffset(line-1);
					if (element.getOffset() <= lineOffset && lineOffset <= (element.getOffset()+element.getLength()))
						return;
					else {
						int nextLineOffset = document.getLineOffset(line);
						problems.add(new Region(lineOffset,nextLineOffset-lineOffset-1));
					}
					
				}*/
			}
		} catch (BadLocationException e) {
			System.err.println("Bad line offset for the erroneous element that breaks the parser.");
		}
	}

	/** Updates the root of the parse tree and gathers the error markers. */
	/*private void parse (){
		int parserBreakLine = 0;
		int docLines = document.getNumberOfLines();
		int docLength = document.getLength();
		String input ="";
		problems.clear();
		sim = null;
		try {
			while (parserBreakLine!=docLines) {
				
		//if (lexer == null) {
			if (parserBreakLine==0) input = document.get();
			else input = document.get(document.getLineOffset(parserBreakLine), docLength-document.getLineOffset(parserBreakLine));
			//System.err.println(input);
			
			StringReader reader = new StringReader(input);
			lexer = new ModestLexer(reader);
		//}
		
		//System.err.println("Document for parsing: " + input);
		
		//if(parser == null)
//			parser = new ModestParser(lexer);
//			parser.setASTNodeClass("ASTWithOffsetInfos");
			parser = new ModestOutlineParser(lexer);
			parser.setActualDocumentOffset(document.getLineOffset(parserBreakLine));//+1?
		
			//parser.simulation();
			//AST parseTree = parser.getAST();
			//System.err.println("Hier vor creatAST call");
			AST parseTree = parser.createAST();
			//problems = parser.problems;
			problems.addAll(parser.problems);
			parserBreakLine = parserBreakLine+parser.getBreakLine();
			
			//Add an error marker if the break line isn't
			//the EOF.
			if (parserBreakLine!=docLines)
				addLineProblem(parserBreakLine);

			//System.err.println("break line: " + parserBreakLine);
			//System.err.println("number of error marker: " + problems.size());
				
			if (parseTree==null) System.err.println("parseTree is null");
			else {
				
				//System.err.println(parseTree.toStringList());
		      
			if(parseTree!=null){
			ModestTreeWalker walker = new ModestTreeWalker();
			if (sim==null) 
				sim =  (Process)walker.simulation(parseTree);
			else 
				sim.addModels(((Process)walker.simulation(parseTree)).getChildren());
			//System.err.println("Root nach dem parsen: "+ sim.toString());
			}
			
			if (sim==null) sim = new Process();
		} 
		}catch (ANTLRException ex){
			System.err.println("Syntax error: " + ex.getMessage());
		}
		catch (Exception e) {
			sim = new Process();
			System.err.println("Exception in ModestSimulation.parse: "+e);
			e.printStackTrace();
		}
	}*/
	
	/** Updates the root of the parse tree, sets currentAST. */
	private void parse (){
		
		String input =document.get();
		sim = null;
		problems.clear();
		try {
			StringReader reader = new StringReader(input);
			lexer = new ModestLexerNew(reader);
			parser = new ModestParserNew(lexer);
			
			
			parser.start();
			variables.clear();
			variables.addAll(parser.getVars());
			processes.clear();
			processes.addAll(parser.getProcesses());
			currentAST = parser.getAST();
			problems = parser.problems;
			warnings = createWarnings(parser.getUnUsedVars());
						  
			if(currentAST!=null){
				//System.err.println(currentAST.toStringList());
				ModestOutlineTreeWalker walker = new ModestOutlineTreeWalker();
				sim =  (Process)walker.start(currentAST);
				isASTValid = walker.OK();
				//if (!isASTValid) System.err.println("Error in walker: " + walker.error());
				//System.err.println("the AST is valid "+ isASTValid + " the walker says OK "+ walker.OK());
			} else isASTValid = false;
			
			
			
			if (sim==null) sim = new Process();
		 
		}catch (ANTLRException ex){
			System.err.println("Syntax error: " + ex.getMessage());
		}
		catch (Exception e) {
			sim = new Process();
			System.err.println("Exception in ModestSimulation.parse: "+e);
			e.printStackTrace();
		}
	}
	
	/** Adds a listener to this program. The listener will be notified
	 * when the program is updated. */
	public void addListener (IModestSimulationListener listener) {listeners.add(listener);}
	/** Removes the listener from this program. If the listener was not
	 * added to the program, nothing happens. */
	public void removeListener (IModestSimulationListener listener) {listeners.remove(listener);}
	/** Notifies all listeners that the program has been updated. */
	private void fireModelChanged () {
		for (Iterator i = listeners.iterator(); i.hasNext(); )
			((IModestSimulationListener) i.next()).simulationChanged(this);
	}

	/**
	 * @return the currentAST or null if the AST isn't valid.
	 */
	public AST getCurrentAST() {
		if (isASTValid)
			return currentAST;
		else return null;
	}
	
	private ArrayList<Region> createWarnings(HashMap<String, Symbol> unUsedVars){
		ArrayList<Region> toReturn = new ArrayList<Region>();
		ArrayList<Symbol> vars = new ArrayList<Symbol>(unUsedVars.values());
		for (int i = 0; i < vars.size(); i++) {
			Symbol var = vars.get(i);
			try {
				int lineOffset = document.getLineOffset(var.getLine()-1);
				int offset = lineOffset + var.getColumn();
				toReturn.add(new Region(offset-1, /*offset +*/ var.getName().length()));			
			} catch (BadLocationException e){
				System.err.println("Bad location for warning");
			}
		}
		
		return toReturn;
	}
	
}
