/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.model;


import modesteditor.core.model.ModestType;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;



/**
 * Model of a MoDeST process.
 *
 */
public class Process extends Model {
	protected List<Process> processes;
	protected List<ModestAction> actions;
	protected List<ModestException> exceptions;
	protected List<ModestVariable> variables;
	protected List<ModestType> structs;
	protected List<Model> childModels;
	
	private static IModelVisitor adder = new Adder();
	private static IModelVisitor remover = new Remover();
	
	private static class Adder implements IModelVisitor {
		
		public void visitAction(ModestAction act, Object argument) {
			((Process) argument).addAction(act);
		}

		public void visitVariable(ModestVariable var, Object argument) {
			((Process) argument).addVariable(var);
		}
		
		public void visitType(ModestType struc, Object argument) {
			((Process) argument).addType(struc);
		}
		
		public void visitModestException(ModestException exc, Object argument) {
			((Process) argument).addModestException(exc);
		}

		
		public void visitProcess(Process proc, Object argument) {
			((Process) argument).addProcess(proc);
		}
		
		public void visitChild(Model model, Object argument) {
			((Process) argument).addChild(model);
		}

	}

	private static class Remover implements IModelVisitor {
		
		public void visitAction(ModestAction act, Object argument) {
			((Process) argument).removeAction(act);
		}

		public void visitVariable(ModestVariable var, Object argument) {
			((Process) argument).removeVariable(var);
		}
		
		public void visitType(ModestType struc, Object argument) {
			((Process) argument).removeType(struc);
		}
		
		public void visitModestException(ModestException exc, Object argument) {
			((Process) argument).removeModestException(exc);
		}

		
		public void visitProcess(Process proc, Object argument) {
			((Process) argument).removeProcess(proc);
			
		}

		public void visitChild(Model model, Object argument) {
			((Process) argument).removeChild(model);
		}
	}
	
	public Process() {
		processes = new ArrayList<Process>();
		actions = new ArrayList<ModestAction>();
		exceptions = new ArrayList<ModestException>();
		variables = new ArrayList<ModestVariable>();
		structs = new ArrayList<ModestType>();
		childModels = new ArrayList<Model>();
	}
	
	public Process(String name, int line, int column) {
		this();
		this.name = name;
		this.line = line;
		this.column = column;
		this.length = name.length();
	}
	
	/**
	 * Returns the child processes
	 * @return the child processes 
	 */
	public List getProcesses() {
		return processes;
	}
	
	/**
	 * Returns the child type definitions
	 * @return the child type definitions
	 */
	public List getTypes() {
		return structs;
	}
	
	/**
	 * Returns the child actions
	 * @return the child actions 
	 */
	public List getActions() {
		return actions;
	}
	
	/**
	 * Returns the child exceptions
	 * @return the child exceptions
	 */
	public List getExceptions() {
		return exceptions;
	}
	
	/**
	 * Returns the child variables
	 * @return the child variables 
	 */
	public List getVariables() {
		return variables;
	}
	
	/**
	 * Returns a list of the child models
	 * @return list of the child models
	 */
	public List getChildren() {
		return childModels;
	}
	
	/**
	 * Add a child process to the list of children
	 * @param proc
	 */
	protected void addProcess(Process proc) {
		proc.parent = this;
		processes.add(proc);
		
	}
	/**
	 * Add a child action to the list of children
	 * @param act
	 */
	protected void addAction(ModestAction act) {
		act.parent = this;
		actions.add(act);
		
	}
	
	/**
	 * Add a child type definition to the list of children
	 * @param struc
	 */
	protected void addType(ModestType struc) {
		struc.parent = this;
		structs.add(struc);
		
	}
	
	/**
	 * Add a child exception to the list of children
	 * @param exc
	 */
	protected void addModestException(ModestException exc) {
		exc.parent = this;
		exceptions.add(exc);
		
	}
	
	/**
	 * Add a child variable to the list of children
	 * @param var
	 */
	protected void addVariable(ModestVariable var) {
		var.parent = this;
		variables.add(var);
		
	}
	
	/**
	 * Add a child model to the list of children
	 * @param model
	 */
	protected void addChild(Model model) {
		model.parent = this;
		childModels.add(model);
	}
	
	public void remove(Model toRemove) {
		toRemove.accept(remover, this);
	}
	
	protected void removeProcess(Process proc) {
		processes.remove(proc);
		
	}
	
	protected void removeAction(ModestAction act) {
		actions.remove(act);
		
	}
	
	protected void removeModestException(ModestException exc) {
		exceptions.remove(exc);
			
	}
	
	protected void removeVariable(ModestVariable var) {
		variables.remove(var);
			
	}
	
	protected void removeType(ModestType struc) {
		structs.remove(struc);
		
	}
	
	protected void removeChild(Model model) {
		childModels.remove(model);
	}
	
	public void add(Model toAdd) {
		toAdd.accept(adder, this);
	}
	
	/** Answer the total number of items the
	 * receiver contains. 
	 */
	public int size() {
		return getActions().size() + getExceptions().size() + getProcesses().size() + getTypes().size() + getVariables().size();
		//return childModels.size();
	}
	
	/*
	 * @see Model#accept(ModelVisitorI, Object)
	 */
	public void accept(IModelVisitor visitor, Object passAlongArgument) {
		visitor.visitProcess(this, passAlongArgument);
	}
	
	/**
	 * Add a list of child actions to the list of children
	 * @param acts
	 */
	public void addActions(List acts) {
		for(int i=0;i<acts.size();i++) addAction((ModestAction)acts.get(i));
	}
	
	/**
	 * Add a list of child exceptions to the list of children
	 * @param excs
	 */
	public void addExceptions(List excs) {
		for (Iterator iter = excs.iterator(); iter.hasNext();)
			addModestException((ModestException) iter.next());
			
		//for (int i=0;i<excs.size();i++) addModestException((ModestException)excs.get(i));
	}
	
	/**
	 * Add a list of child processes to the list of children
	 * @param procs
	 */
	public void addProcesses(List procs) {
		for (Iterator iter = procs.iterator(); iter.hasNext();) 
			 addProcess((Process) iter.next());
	}
	
	/**
	 * Add a list of child type definitions to the list of children
	 * @param strucs
	 */
	public void addTypes(List strucs) {
		for (Iterator iter = strucs.iterator(); iter.hasNext();) 
			addType((ModestType)iter.next());
	}
	
	/**
	 * Add a list of child variables to the list of children
	 * @param vars
	 */
	public void addVariables(List vars) {
		for (Iterator iter = vars.iterator(); iter.hasNext();) 
			addVariable((ModestVariable)iter.next());
	}
	
	/**
	 * Add a list of models to the list of children
	 * @param models
	 */
	public void addModels(List models) {
		for (Iterator iter = models.iterator(); iter.hasNext();) 
			addChild((Model) iter.next());	
	}
	
}
	
	

