/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.modest;

import modesteditor.core.ModestEditor;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.TextPresentation;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ContextInformation;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationPresenter;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * MoDeST completion processor
 *
 */
public class ModestCompletionProcessor implements IContentAssistProcessor {
	
	/**
	 * Simple content assist tip closer. The tip is valid in a range
	 * of 5 characters around its popup location.
	 */
	protected static class Validator implements IContextInformationValidator, IContextInformationPresenter {

		protected int fInstallOffset;

		/*
		 * @see IContextInformationValidator#isContextInformationValid(int)
		 */
		public boolean isContextInformationValid(int offset) {
			return Math.abs(fInstallOffset - offset) < 5;
		}

		/*
		 * @see IContextInformationValidator#install(IContextInformation, ITextViewer, int)
		 */
		public void install(IContextInformation info, ITextViewer viewer, int offset) {
			fInstallOffset= offset;
		}
		
		/*
		 * @see org.eclipse.jface.text.contentassist.IContextInformationPresenter#updatePresentation(int, TextPresentation)
		 */
		public boolean updatePresentation(int documentPosition, TextPresentation presentation) {
			return false;
		}
	}

	protected final static String[] fgProposals=
		{ 
		"action",
		"alt", 
		"bool",
		"break",
		"by",
		"catch",
		"clock",
		"const",
		"do", 
		"exception",
		"Exponential",
		"extend",
		"extern",
		"float",
		"hide",
		"impatient",
		"int",
		"Normal",
		"palt", 
		"par", 
		"patient",
		"prefix",
		"try", 
		"when", 
		"process", 
		"range", 
		"relabel",
		"return",
		"set", 
		"struct",
		"throw",
		"typedef",
		"Uniform",
		"urgent"
		 }; 

	protected IContextInformationValidator fValidator= new Validator();

	/**
	 * Returns an array which contains the completion proposals. It computes out of the 
	 * entered prefix the matching keyword.
	 * In the future, may implement the completion of declared variables.
	 */ 
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer, int documentOffset) {
		/* First try, gets every keyword into the proposals
		  		 
		ICompletionProposal[] result= new ICompletionProposal[fgProposals.length];
		for (int i= 0; i < fgProposals.length; i++) {
			IContextInformation info= new ContextInformation(fgProposals[i], MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.ContextInfo.pattern"), new Object[] { fgProposals[i] })); //$NON-NLS-1$
			result[i]= new CompletionProposal(fgProposals[i], documentOffset, 0, fgProposals[i].length(), null, fgProposals[i], info, MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { fgProposals[i]})); //$NON-NLS-1$
		}
		return result;
		
		*/
		
		/* Use the above and put the replacementString (first argument) without the prefix,
		 * that should already do the trick.
		 *  TODO implement variable completion
		 */
		
		List result= new ArrayList();
		String prefix= extractPrefix(viewer, documentOffset);
		
		for (int i= 0; i < fgProposals.length; i++) {
			if (fgProposals[i].startsWith(prefix)) {
				String proposal = fgProposals[i].substring(prefix.length());
				//System.out.println(proposal);
				//IContextInformation info= new ContextInformation(fgProposals[i], MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.ContextInfo.pattern"), new Object[] { fgProposals[i] }));
				result.add(new CompletionProposal(proposal, documentOffset, 0, proposal.length(), null, fgProposals[i], null, MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { fgProposals[i]})));
			}
		}
		
		ArrayList<String> vars = ModestEditor.getEditor().getSimulation().getVariables();
		for (int i= 0; i < vars.size(); i++) {
			String var = vars.get(i);
			if (var.startsWith(prefix)) {
				String proposal = var.substring(prefix.length());
				//System.out.println(proposal);
				//IContextInformation info= new ContextInformation(var, MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.ContextInfo.pattern"), new Object[] { var }));
				result.add(new CompletionProposal(proposal, documentOffset, 0, proposal.length(), null, var, null, MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { var })));
			}
		}
		
		ArrayList<String> procs = ModestEditor.getEditor().getSimulation().getProcesses();
		for (int i = 0; i < procs.size(); i++) {
			String proc = procs.get(i) + "()";
			if (proc.startsWith(prefix)) {
				String proposal = proc.substring(prefix.length());
				result.add(new CompletionProposal(proposal, documentOffset, 0, proposal.length(), null, proc, null, MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { proc })));
			}
		}
		
		return (ICompletionProposal[])result.toArray(new ICompletionProposal[result.size()]);
	}
	
	/**
	 * Extracts the prefix used for determining the possible completions
	 * from the viewer's document. The default implementation returns the String from
	 * offset backwards that forms a java identifier.
	 *
	 * @param viewer the viewer
	 * @param offset offset into document
	 * @return the prefix to consider
	 */
	protected String extractPrefix(ITextViewer viewer, int documentOffset) {
		int i= documentOffset;
		IDocument document= viewer.getDocument();
		if (i > document.getLength())
			return ""; 

		try {
			while (i > 0) {
				char ch= document.getChar(i - 1);
				if (!Character.isJavaIdentifierPart(ch))
					break;
				i--;
			}

			return document.get(i, documentOffset - i);
		} catch (BadLocationException e) {
			return ""; 
		}
	}
	
	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public IContextInformation[] computeContextInformation(ITextViewer viewer, int documentOffset) {
		IContextInformation[] result= new IContextInformation[5];
		for (int i= 0; i < result.length; i++)
			result[i]= new ContextInformation(
				MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.ContextInfo.display.pattern"), new Object[] { new Integer(i), new Integer(documentOffset) }),  //$NON-NLS-1$
				MessageFormat.format(ModestEditorMessages.getString("CompletionProcessor.ContextInfo.value.pattern"), new Object[] { new Integer(i), new Integer(documentOffset - 5), new Integer(documentOffset + 5)})); //$NON-NLS-1$
		return result;
	}
	
	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public char[] getCompletionProposalAutoActivationCharacters() {
		//return new char[] { '.', '(' };
		return new char[] { /*':'*/ };
	}
	
	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public char[] getContextInformationAutoActivationCharacters() {
		//return new char[] { '#' };
		return new char[] {};
	}
	
	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public IContextInformationValidator getContextInformationValidator() {
		return fValidator;
	}
	
	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public String getErrorMessage() {
		return null;
	}

}
