/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing an alt statement in the step simulation tree.
 */
public class AltNode extends SimulationNode {

	/** The active child, only one since sequential */
	protected SimulationNode activeChild;
	
	/**
	 * Default constructor.
	 * @param line
	 * @param column
	 * @param parent
	 */
	public AltNode(int line, int column, SimulationNode parent) {
		super(line, column);
				
		setParent(parent);
		parent.addChild(this);
		activeChild = null;
		length = 3;
	}
	
	/**
	 * Sets the node active and all children in the children list
	 */
	@Override
	public void setActive() {
		if (activeChild == null && !children.isEmpty())
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				child.setActive();
			}*/
			for (int i = 0; i < children.size(); i++) {
				children.get(i).setActive();
			}
				
		super.setActive();
	}
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (child != activeChild) 
			System.err.println("Wrong argument to childIsFinished for AltNode: " + this.toString());
		else 
			isFinished();
			
	}

	@Override
	public void isFinished() {
		//if (isActive()){
			super.isFinished();
			activeChild = null;
			parent.childIsFinished(this);
		//}
	}
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	/**
	 * Method collects all children if not a child is already active
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive())
			if (activeChild!=null)
				return activeChild.collectTransitions();
			else {
				ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
				/*for (Iterator iter = children.iterator(); iter.hasNext();) {
					SimulationNode child = (SimulationNode) iter.next();
					trans.addAll(child.collectTransitions());
				}*/
				for (int i = 0; i < children.size(); i++) {
					trans.addAll(children.get(i).collectTransitions());
				}
				return trans;
			}	
		else 
			return null;
	}

	
	@Override
	public List<ParTransition> collectParTransitions() {
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			if (activeChild!=null){
				List<ParTransition> tmpTrans = activeChild.collectParTransitions();
				HashSet<String> containedLetters = new HashSet<String>();
				for (int i = 0; i < tmpTrans.size(); i++) {
					ParTransition actParTrans = tmpTrans.get(i);
					trans.add(actParTrans);
					String repLetter = actParTrans.getRepresentingLetter();
					if (repLetter!="break" && repLetter!="tau" && repLetter!="stop") 
						containedLetters.add(repLetter);
				}
				HashSet<String> myAlphabet = new HashSet<String>();
				myAlphabet.addAll((ArrayList<String>)getAlphabet().clone());
				for (Iterator iter = myAlphabet.iterator(); iter.hasNext();) {
					String element = (String) iter.next();
					/* TODO */
					// Don't know if this works since the transitions list is empty
					if (!containedLetters.contains(element))
						trans.add(new ParTransition(this.parent, element));
				}
			} else {
				for (int i = 0; i < children.size(); i++) {
					trans.addAll(children.get(i).collectParTransitions());
					/*if (!children.get(i).collectParTransitions().isEmpty())
						System.err.println("ParTrans from " +this.toString() + " " 
								+children.get(i).collectParTransitions().get(0).printTransition());*/
				}
			}
		}
		return trans;
	}
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet!=null && alphabet.isEmpty())
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				alphabet.addAll(child.getAlphabet());
			}*/
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child =children.get(i);
				// Since the children are not necessary activated we 
				// generate the alphabet by hand.
				child.generateAlphabet();
				alphabet.addAll(child.getAlphabet());
			}
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	/**
	 * No transition to be taken, only child can be consumed.
	 */
	@Override
	public void takeTransition() {
		System.err.println("Cannot take transition in AltNode: "+this.toString());
	}
	
	@Override
	public void releaseTransitions() {
		if (isActive()){
			isActive=false;
			activeChild = null;
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				child.releaseTransitions();
			}*/
			for (int i = 0; i < children.size(); i++) {
				children.get(i).releaseTransitions();
			}
		}
	}
	
	public void notificationOfChildActivity(SimulationNode child){
		if (children.contains(child)){ 
			activeChild=child;
			parent.notificationOfChildActivity(this);
			/** TODO check if this works for all suclasses */
			for (int i = 0; i < children.size(); i++) {
				if (child != children.get(i))
					children.get(i).releaseTransitions();
			}
		}
	}
	
	@Override
	public ArrayList<SimulationNode> getParentList() {
		ArrayList<SimulationNode> parentList = new ArrayList<SimulationNode>();
		parentList.add(this);
		return parentList;
	}

}
