/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing an alternative in the step simulation tree.
 */
public class Alternative extends SimulationNode {

	private SimulationNode activeChild;
	//* Weight of an alternative for a palt node
	private double weight;
	private String assignment;

	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param parent
	 */
	public Alternative(int line, int column, SimulationNode parent) {
		super(line, column);
		this.parent = parent;
		parent.addChild(this);
		length = 2;
		activeChild = null;
		assignment = "";
		weight=0;
	}
	
	/**
	 * Constructor for an alternative of a palt node including the weight.
	 * @param line
	 * @param column
	 * @param parent
	 * @param weight
	 */
	public Alternative(int line, int column, SimulationNode parent, double weight) {
		super(line, column);
		this.parent = parent;
		this.weight = weight;
		parent.addChild(this);
		Double myWeight = new Double(weight); 
		length = 2 + Integer.toString(myWeight.intValue()).length();
		activeChild = null;
		assignment = "";
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (children.contains(child))  isFinished(); 
		else System.err.println("Wrong argument to childIsFinished in Alternative, it can only have one child: " + this.toString());

	}
	
	@Override
	public void isFinished() {
		super.isFinished();
		activeChild=null;
		parent.childIsFinished(this);
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()) {
			if (activeChild!=null)
				return activeChild.collectTransitions();
			else {
				/*ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
				trans.add(this);
				return ((List<SimulationNode>) trans);*/
				ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
				for (int i = 0; i < children.size(); i++) {
					trans.addAll(children.get(i).collectTransitions());
				}
				return trans;
			}
		} else
			return null;
	}
	
	@Override
	public List<ParTransition> collectParTransitions() {
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive() && activeChild != null){
			List<ParTransition> tmpTrans = activeChild.collectParTransitions();
			HashSet<String> containedLetters = new HashSet<String>();
			for (int i = 0; i < tmpTrans.size(); i++) {
				ParTransition actParTrans = tmpTrans.get(i);
				trans.add(actParTrans);
				if (actParTrans.getRepresentingLetter()!="break"
					&& actParTrans.getRepresentingLetter()!="stop"
						&& actParTrans.getRepresentingLetter()!="tau") 
					containedLetters.add(actParTrans.getRepresentingLetter());
			}
			HashSet<String> myAlphabet = new HashSet<String>();
			myAlphabet.addAll((ArrayList<String>)getAlphabet().clone());
			for (Iterator iter = myAlphabet.iterator(); iter.hasNext();) {
				String element = (String) iter.next();
				/* TODO */
				// Don't know if this works since the transitions list is empty
				if (!containedLetters.contains(element))
					trans.add(new ParTransition(this.parent, element));
			}
		} else {
			if (alphabet.isEmpty())
				generateAlphabet();
			HashSet<String> alph = new HashSet<String>(alphabet);
			for (Iterator iter = alph.iterator(); iter.hasNext();) {
				String letter = (String) iter.next();
				trans.add(new ParTransition(this.parent, letter));
			}
		}
		return trans;
	}

	/**
	 * Generates the alphabet as the  union of all children, which should be only one child in fact.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet!=null && alphabet.isEmpty())
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				// Child is  not obviously active so generate the alphabet by hand
				child.generateAlphabet();
				alphabet.addAll(child.getAlphabet());
			}*/
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child = children.get(i);
				child.generateAlphabet();
				alphabet.addAll(child.getAlphabet());
			}
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#releaseTransitions()
	 */
	@Override
	public void releaseTransitions() {
		if (isActive())
			if (activeChild!=null) {
				isActive=false;
				activeChild.releaseTransitions();
				activeChild = null;
			} else 
				isActive=false;
		
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	@Override
	public void takeTransition() {
			
		// Only taken in case of a PaltTransition
		setActive();
		parent.notificationOfChildActivity(this);
		if (children.isEmpty())
			isFinished();
	}
	
	@Override
	public void setActive() {
		for (int i = 0; i < children.size(); i++) {
			children.get(i).setActive();
		}
		if (!children.isEmpty()) 
			activeChild = children.get(0);
		super.setActive();
	}

	@Override
	public void notificationOfChildActivity(SimulationNode child) {
		activeChild = child;
		parent.notificationOfChildActivity(this);
	}
	
	/**
	 * @return the weight as a float
	 */
	public double getWeight() {
		/*if (weight == "") return 0;
		return Float.valueOf(weight);*/
		return weight;
	}
	
	@Override
	public String printTransition() {
		return activeChild.printTransition();
	}

	/**
	 * @param assignment the assignment to set
	 */
	public void setAssignment(String assignment) {
		this.assignment = assignment;
		
	}

	/**
	 * @return the assignment
	 */
	public String getAssignment() {
		return assignment;
	}
	
	
}
