/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.text.IDocument;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing a palt statement.
 *
 */
public class PaltNode extends AltNode {
	
	private ActionNode actionGuard;
	private double weightSum;

	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param parent
	 */
	public PaltNode(int line, int column, SimulationNode parent) {
		super(line, column, parent);
		actionGuard = null;
		length = 4;
		weightSum = 0;
	}

	
	/**
	 * Method adds a child to the list if it is an Alternative. If it is an 
	 * ActionNode it is set a the actionGuard of the statement.
	 * @param child to add
	 */
	@Override
	public void addChild(SimulationNode child) {
		if (child instanceof ActionNode) {
			actionGuard = (ActionNode) child;
			guard = child.getGuard();
		}
		else {
			super.addChild(child);
			if (child instanceof Alternative)
				weightSum += ((Alternative)child).getWeight();
		}
		
	}
	
	@Override
	public void setActive() {
		
		if (actionGuard != null){
			actionGuard.setActive();
			//activeChild=actionGuard;
		}
		
		this.isActive=true;
		generateAlphabet();
	}
	
	/**
	 * Method generates the alphabet from the actionGuard and all the 
	 * alternatives.
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet!=null && alphabet.isEmpty()){
			if (actionGuard != null){
				actionGuard.generateAlphabet();
				alphabet.addAll(actionGuard.getAlphabet());
			}
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child =children.get(i);
				// Since the children are not necessarily activated we 
				// generate the alphabet by hand.
				child.generateAlphabet();
				alphabet.addAll(child.getAlphabet());
			}
		}
	}
	
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()){
		/*if (activeChild.equals(actionGuard))
			return actionGuard.collectTransitions();
		else {
			ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
			for (int i = 0; i < children.size(); i++) {
				trans.addAll(children.get(i).collectTransitions());
			}
			return trans;
		}*/
			if (activeChild == null){
				ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
				trans.add(this);
				return trans;
			} else return activeChild.collectTransitions();
			
		} else return null;
	}
	
	
	/**
	 * If a child is active the ParTransitions from this child are collected.
	 * Otherwise a ParTransition is created from the actionguard and the
	 * probabilistic transitions are added to this ParTransition.
	 */
	@Override
	public List<ParTransition> collectParTransitions() {
		
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			
			if (activeChild!=null){
				
				List<ParTransition> tmpTrans = activeChild.collectParTransitions();
				
				HashSet<String> containedLetters = new HashSet<String>();
				for (int i = 0; i < tmpTrans.size(); i++) {
					ParTransition actParTrans = tmpTrans.get(i);
					trans.add(actParTrans);
					String repLetter = actParTrans.getRepresentingLetter();
				
					if (repLetter!="break" && repLetter!="tau" && repLetter!="stop") 
						containedLetters.add(repLetter);
				}
				HashSet<String> myAlphabet = new HashSet<String>();
				myAlphabet.addAll((ArrayList<String>)getAlphabet().clone());
				
				for (Iterator iter = myAlphabet.iterator(); iter.hasNext();) {
					String element = (String) iter.next();
					/* TODO */
					// Don't know if this works since the transitions list is empty
					if (!containedLetters.contains(element))
						trans.add(new ParTransition(this.parent, element));
				}
			} else {
				/*for (int i = 0; i < children.size(); i++) {
					trans.addAll(children.get(i).collectParTransitions());}*/
				ParTransition actTrans = new ParTransition(this.parent, actionGuard.getName());//getAlphabet().get(0));
				actTrans.addTransition(this);
				actTrans.addPaltTransitionList(getPaltTransitions());
				if (!actionGuard.getAssignment().equals("")) actTrans.setAssignment(actionGuard.getAssignment());
				trans.add(actTrans);
								
			}
		}
		
		return trans;
	}
	
	@Override
	public void releaseTransitions() {
		actionGuard.releaseTransitions();
		super.releaseTransitions();
	}
	
	/**
	 * Method activates the alternatives if the actionGuard has been taken or 
	 * finishes the PaltNode if an Alternative is done.
	 * @param child that is finished
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (child != activeChild && !child.equals(actionGuard)) 
			System.err.println("Wrong argument to childIsFinished for PaltNode: " + this.toString());
		else { 
			if (child.equals(actionGuard) && !children.isEmpty())
				activeChild=null;
			else
				isFinished();
		}		
		//super.childIsFinished(child);
	}
	
	@Override
	public String printTransition() {
		//String trans = actionGuard.getName() + " (probabilistic transition following)";
		String trans = actionGuard.printTransition();
		if (getPaltTransitions().size()==1) trans += " " + getPaltTransitions().get(0).printTransition();
		else if (!getPaltTransitions().isEmpty()) trans += " (probabilistic transition following)";
		return trans;
	}
	
	/**
	 * @return the sum of all weights (needed to compute probabilities).
	 */
	public double getWeightSum() {
		return weightSum;
	}
	
	/**
	 * Method for generating the list of PaltTransition of the node
	 * @return the list of PaltTransitions of the node
	 */
	public ArrayList<PaltTransition> getPaltTransitions(){
		ArrayList<PaltTransition> trans = new ArrayList<PaltTransition>();
		for (int i = 0; i < children.size(); i++) {
			Alternative alternative = (Alternative)children.get(i);
			trans.add(new PaltTransition(alternative.getLine(), alternative.getColumn(), alternative.getWeight()/getWeightSum(), alternative));
		}
		return trans;
	}
	
	@Override
	public void notificationOfChildActivity(SimulationNode child) {
		if (children.contains(child)){
			activeChild = child;
			parent.notificationOfChildActivity(this);
		}
	}
	
	@Override
	public void takeTransition() {
		parent.notificationOfChildActivity(this);
		actionGuard.takeTransition();
	}
	
	@Override
	public ArrayList<IMarker> createSimHighlighting(IResource resource, IDocument document, boolean isPalt) {
		ArrayList<IMarker> markers = new ArrayList<IMarker>();
		try {
			if (isPalt){
				int line = getLine()-1;
				int column = getColumn();
				if (line+1 != 0) {
					int offset = document.getLineOffset(line) + column -1;

					IMarker marker = resource.createMarker(IMarker.PROBLEM);//("modesteditor.core.simulationmarker");//IMarker.TEXT
					marker.setAttribute(IMarker.MESSAGE, "active part of the simulation");
					marker.setAttribute(IMarker.CHAR_START, offset);
					marker.setAttribute(IMarker.CHAR_END, offset+length);
					//marker.setAttribute("isParallel", false);

					markers.add(marker);
				}
			} else {

				int line = actionGuard.getLine()-1;
				int column = actionGuard.getColumn();
				int offset = document.getLineOffset(line) + column -1;

				IMarker marker = resource.createMarker(IMarker.PROBLEM);//("modesteditor.core.simulationmarker");//IMarker.TEXT
				marker.setAttribute(IMarker.MESSAGE, "active part of the simulation");
				marker.setAttribute(IMarker.CHAR_START, offset);
				marker.setAttribute(IMarker.CHAR_END, offset+actionGuard.getLength());
				//marker.setAttribute("isParallel", false);

				markers.add(marker);
			}
		} catch (Exception e){
			System.err.println("Could not create marker in "+ this.toString());
		} 

		if (parent != null)
			markers.addAll(parent.createSimHighlighting(resource, document, isPalt));

		return markers;
	}	
	
	@Override
	public String getName() {
		return actionGuard.getName();
	}
	
}
