/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing a par statement in the step simulation tree.
 */
public class ParNode extends SimulationNode {

	//* Common alphabet of the par statement
	private HashSet<String> commonAlphabet;
	
	//* List of the still active children
	private ArrayList<SimulationNode> activeChildren;
	
	/** 
	 * Flag telling the collectTransitions method if it has to 
	 * save the number of alternatives per ParTransition.
	 */
	private boolean firstRun;
	/**
	 * Is there for remembering how much alternatives should be 
	 * present per letter in the common alphabet.
	 */
	private HashMap<String, Integer> commonAlphabetAlternativeCount;
	
	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param parent
	 */
	public ParNode(int line, int column, SimulationNode parent) {
		super(line, column);
		this.parent=parent;
		parent.addChild(this);
		activeChildren = new ArrayList<SimulationNode>();
		commonAlphabet = new HashSet<String>();
		length = 3;
		firstRun = true;
		commonAlphabetAlternativeCount = new HashMap<String, Integer>();
	}
	
	/**
	 * Sets all children active and register them in the activeChildren List.
	 * Builds the common alphabet.
	 * super.setActive() called last, so that the alphabet of the children is 
	 * already generated.
	 */
	@Override
	public void setActive() {
		if (!children.isEmpty()){
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				child.setActive();
				activeChildren.add(child);
			}*/
			activeChildren.clear();
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child = children.get(i);
				child.setActive();
				activeChildren.add(child);
			}
		}
		buildCommonAlphabet();
		super.setActive();
	}

	/**
	 * Method removes the finished child from the list of active children. 
	 * If this list is empty, the ParNode is finished too.
	 * @param child that is finished
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (!activeChildren.contains(child)) 
			System.err.println("Wrong argument to childIsFinished for ParNode: " + this.toString());
		else {
			activeChildren.remove(child);
			// No active children left, so we are done
			if (activeChildren.isEmpty()) isFinished();
		}
	}

	@Override
	public void isFinished() {
		super.isFinished();
		parent.childIsFinished(this);
	}
	
	/**
	 * Method returns the collected parallel transitions from collectParTransitions.
	 * This method is responsible for switching from a collectTransitions call tree 
	 * to a collectParTransitions call tree. 
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()){
						
			List<ParTransition> transens = collectParTransitions();
									
			return new ArrayList<SimulationNode>(transens);
		}
		else 
			return null;
	}
	
	/**
	 * Method first collects the ParTransitions from it's active children. 
	 * These parallel transitions are then organized into single parallel 
	 * transitions respecting the action they are representing, the enabledness 
	 * of such a transition and the different alternatives of a node. 
	 */
	@Override
	public List<ParTransition> collectParTransitions() {
		
		HashSet<String> cleanCommonAlphabet = new HashSet<String>();
		cleanCommonAlphabet.addAll(commonAlphabet);
		
			
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		// Check how to combine the children.
		
		HashMap<String, ArrayList<ArrayList<ParTransition>>> letterToTransMap = 
			new HashMap<String, ArrayList<ArrayList<ParTransition>>>();
		//for (int i = 0; i < activeChildren.size(); i++) {
		
		// Need disabled trans too
		for (int i = 0; i < children.size(); i++) {
			HashSet<String> childCommonAlphabet = new HashSet<String>();
			HashMap<String, ArrayList<ParTransition>> childLetterToTransMap = 
				new HashMap<String, ArrayList<ParTransition>>();
			SimulationNode child = children.get(i);
			
			List<ParTransition> childTrans = child.collectParTransitions();
			
			for (int j = 0; j < childTrans.size(); j++) {
				ParTransition actTrans = childTrans.get(j);
				String letter = actTrans.getRepresentingLetter();
				
				// Fill the set of letters in the common alphabet that are 
				//already represented
				childCommonAlphabet.add(letter);
				// Garantee a mapping from letters in the common alphabet to transitions
				if (childLetterToTransMap.containsKey(letter)){
					childLetterToTransMap.get(letter).add(actTrans);
				}
				else {
					ArrayList<ParTransition> newParList = new ArrayList<ParTransition>();
					newParList.add(actTrans);
					childLetterToTransMap.put(letter, newParList);
				}
			}
			
			// Iterate over the collected letters from the common alphabet and build 
			// ParTransitions for the out singled letter.
			for (Iterator iter = childCommonAlphabet.iterator(); iter.hasNext();) {
				String letter = (String) iter.next();

				// If the letter is in the common alphabet save the transition 
				// for later use
				if (cleanCommonAlphabet.contains(letter)){

					if (letterToTransMap.containsKey(letter)){
						letterToTransMap.get(letter).add(childLetterToTransMap.get(letter));

					} else {
						ArrayList<ArrayList<ParTransition>> tmpList = 
							new ArrayList<ArrayList<ParTransition>>();
						tmpList.add(childLetterToTransMap.get(letter));
						letterToTransMap.put(letter, tmpList);
					}
				} else {
					// If the letter isn't in the common alphabet
					// add the transition to the final transition list.
					
					ArrayList<ParTransition> nonCommonTrans = childLetterToTransMap.get(letter);
					for (int j = 0; j < nonCommonTrans.size(); j++) {
						ParTransition transToCheck = nonCommonTrans.get(j);
						if (transToCheck.isEnabled())
							trans.add(transToCheck);
					}
				}
			} 
		}


		// Got all transitions from the children, now process the common alphabet to put 
		// the ParTransitions together.
		for (Iterator iter = cleanCommonAlphabet.iterator(); iter.hasNext();) {
			String letter = (String) iter.next();

			// Only work on letters that have transitions.
			if (letterToTransMap.containsKey(letter)){
				
				ArrayList<ArrayList<ParTransition>> transMatrix = letterToTransMap.get(letter);

				// Working on the matrix of possibilities such that all possible combinations
				// from different parent nodes ar put together.
				ArrayList<ArrayList<ParTransition>> toConvertInFinalParTrans = new ArrayList<ArrayList<ParTransition>>();
				for (int i = 0; i < transMatrix.size(); i++) {
					ArrayList<ParTransition> transRow = transMatrix.get(i);

					ArrayList<ArrayList<ParTransition>> tmpToConvertInFinalParTrans = 
						new ArrayList<ArrayList<ParTransition>>();
					for (int j = 0; j < transRow.size(); j++) {
						ParTransition actTrans = transRow.get(j);
						
						ArrayList<ArrayList<ParTransition>> convertListCopy = new ArrayList<ArrayList<ParTransition>>();
						for (int k = 0; k < toConvertInFinalParTrans.size(); k++) {
							convertListCopy.add((ArrayList<ParTransition>)toConvertInFinalParTrans.get(k).clone());
						}
						
						if (convertListCopy.isEmpty()){
							ArrayList<ParTransition> tmpTransList = new ArrayList<ParTransition>();
							tmpTransList.add(actTrans);
							convertListCopy.add(tmpTransList);
						} else {
							for (int k = 0; k < convertListCopy.size(); k++) {
								convertListCopy.get(k).add(actTrans);

							}
						}
						tmpToConvertInFinalParTrans.addAll(convertListCopy);
					}
					toConvertInFinalParTrans = tmpToConvertInFinalParTrans;
				}
				
				// The sets are built, now build the final ParTransitions
				for (int i = 0; i < toConvertInFinalParTrans.size(); i++) {
					ArrayList<ParTransition> toConvert = toConvertInFinalParTrans.get(i);
					
					// Here the PaltTransitions are built out of the matrix
					ArrayList<PaltTransition> paltTransToAdd = buildPaltTransitions(toConvert);
					
					ArrayList<SimulationNode> transToAddToParTrans =
						new ArrayList<SimulationNode>();
					boolean transEnabled = true;
					String assignment = "";
					for (int j = 0; j < toConvert.size(); j++) {
						ParTransition transToConvert = toConvert.get(j);
						if (transToConvert.isEnabled()){
							transToAddToParTrans.addAll(transToConvert.getTransitions());
							if (!transToConvert.getAssignment().equals(""))
								assignment += assignment.equals("") ? transToConvert.getAssignment() : ", " + transToConvert.getAssignment(); 
						}
						else transEnabled=false;
					}
					if (transEnabled) {
						ParTransition toAdd = new ParTransition(this, letter, transToAddToParTrans);
						toAdd.addPaltTransitionList(paltTransToAdd);
						if (!assignment.equals("")) toAdd.setAssignment(assignment);
						trans.add(toAdd);
					} 
				}
			} 
			
			// TODO check whether the disabled trans are needed!
			
			/*else {
				// Add a disabled Trans
				trans.add(new ParTransition(this.parent, letter));
								
			}*/
		}
		return trans;
	}
	
	/**
	 * Method building the parallel PaltTransitions for the ParTransitions that have 
	 * probabilistic alternatives.
	 * @param trans List of the transitions with the p-alternatives.
	 * @return the list of PaltTransitions for a specific letter of the common 
	 * alphabet.
	 */
	public ArrayList<PaltTransition> buildPaltTransitions(ArrayList<ParTransition> trans) {
		
		ArrayList<PaltTransition> paltTrans = new ArrayList<PaltTransition>();		
		
		for (int i = 0; i < trans.size(); i++) {
			ArrayList<PaltTransition> tmpPaltTrans = trans.get(i).getPaltTransitions();
			if (paltTrans.isEmpty()) paltTrans = tmpPaltTrans; 
			else {
				ArrayList<PaltTransition> paltTransToCopy = new ArrayList<PaltTransition>();
				for (int j = 0; j < tmpPaltTrans.size(); j++) {
					PaltTransition tmpPalt = tmpPaltTrans.get(j);
					for (int k = 0; k < paltTrans.size(); k++) {
						PaltTransition origPaltTrans = paltTrans.get(k);
						PaltTransition transAdded = 
							new PaltTransition(tmpPalt.getLine(), tmpPalt.getColumn(), 
									tmpPalt.getProbability(), tmpPalt.getPAlternatives());
						transAdded.addAlternative(origPaltTrans);
						paltTransToCopy.add(transAdded);
					}
				}
				if (!paltTransToCopy.isEmpty())
					paltTrans = paltTransToCopy;
			}
		}
		
		return paltTrans;
	}
	
	/**
	 * Builds the common alphabet for this par construct.
	 */
	private void buildCommonAlphabet(){
		
		if (commonAlphabet.isEmpty())
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child = children.get(i);
				ArrayList<String> childAlphabet = (ArrayList<String>)child.getAlphabet().clone();
				for (int j = 0; j < children.size(); j++) {
					if (i == j) continue;
					SimulationNode child2 = children.get(j);
					ArrayList<String> childAlphabet2 = (ArrayList<String>)child2.getAlphabet().clone();
					// Retain only the intersection of both sets 
					childAlphabet2.retainAll(childAlphabet);
					// Add the intersection, assured only one instance of every letter 
					// due to the set property
					commonAlphabet.addAll(childAlphabet2);
				}
			}
		
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet!=null && alphabet.isEmpty()) {
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				alphabet.addAll(child.getAlphabet());
			}*/
			for (int i = 0; i < children.size(); i++) {
				alphabet.addAll(children.get(i).getAlphabet());
			}
		}
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#releaseTransitions()
	 */
	@Override
	public void releaseTransitions() {
		if (isActive()){
			isActive=false;
			/*for (Iterator iter = activeChildren.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				child.releaseTransitions();
			}*/
			for (int i = 0; i < activeChildren.size(); i++) {
				activeChildren.get(i).releaseTransitions();
			}
			activeChildren.clear();
		}
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	@Override
	public void takeTransition() {}
	
	@Override
	public void notificationOfChildActivity(SimulationNode child) {
		parent.notificationOfChildActivity(this);
	}
	
	@Override
	public ArrayList<SimulationNode> getParentList() {
		ArrayList<SimulationNode> parentList = new ArrayList<SimulationNode>();
		parentList.add(this);
		return parentList;
	}

}
