/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.text.IDocument;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing a sequential process in the step simulation tree.
 * Possibly the root node of the simulation since the whole simulation is 
 * a sequential process.
 */
public class SequentialProcess extends SimulationNode {

	
	//* The active child, only one since sequential
	private SimulationNode activeChild;
	
	private boolean stopped;
	
	private boolean exceptionAbort;
	
	private ArrayList<String> alphabetExtension;
	
	private ArrayList<String> alphabetToHide;
	
	private HashMap<String, String> relabeling;
	
	/**
	 * Default constructor.
	 * @param line
	 * @param column
	 * @param parent
	 */
	public SequentialProcess(int line, int column, SimulationNode parent) {
		super(line, column);
				
		setParent(parent);
		if (parent!=null) parent.addChild(this);
		activeChild = null;
		length = 0;
		stopped = false;
		exceptionAbort = false;
		alphabetExtension = new ArrayList<String>();
		alphabetToHide = new ArrayList<String>();
		relabeling = new HashMap<String, String>();
	}
	
	/**
	 * Constructor for the root node which is definitively a sequential process.
	 */
	public SequentialProcess() {
		super();
		setParent(null);
	}
	
	/**
	 * Check if this is the root node.
	 * @return true if it is the root node
	 */
	public boolean isRoot() {
		return (parent==null);
	}
	
	/**
	 * Sets the node active and the first child in the children list
	 */
	@Override
	public void setActive() {
		
		if (activeChild == null && !children.isEmpty()) {
			activeChild=children.get(0);
			activeChild.setActive();
		}
		super.setActive();
	}
	
	
	/**
	 * If the child is not the last child in the children list, the next one is set active.
	 * Otherwise the process is finished.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (child != activeChild) {
			System.err.println("Wrong argument to childIsFinished for SequentialProcess: " + this.toString());
			return;
		}
		else {
			if (children.indexOf(activeChild)<(children.size()-1)) { 
				activeChild = children.get(children.indexOf(activeChild)+1);
				activeChild.setActive();
			}
			else isFinished();
		}
	}

	@Override
	public void releaseTransitions() {
		if (isActive()){
			isActive=false;
			activeChild.releaseTransitions();
			activeChild = null;
			//activeChildIt = children.iterator();
		}
	}
	
	@Override
	public void isFinished()  {
		super.isFinished();
		activeChild = null;
		if (parent!=null) parent.childIsFinished(this);
		
	}
	
	/**
	 * Method collects the transitions from the active child.
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()) {
			return relabel(activeChild.collectTransitions());
		} else
			return new ArrayList<SimulationNode>();//null;
	}
	

	/**
	 * Method collects the transitions from the active child.
	 */
	@Override
	public List<ParTransition> collectParTransitions(){
		List<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			List<ParTransition> toRelabel = activeChild.collectParTransitions();
			
			// Do the relabeling here, with the hiding. These actions should be renamed in tau
			trans = parRelabel(toRelabel);
			//trans = toRelabel;
			HashSet<String> containedLetters = new HashSet<String>();
			for (int i = 0; i < trans.size(); i++) {
				ParTransition actParTrans = trans.get(i);
				if (actParTrans.getRepresentingLetter()!="break"
					&& actParTrans.getRepresentingLetter()!="stop"
						&& actParTrans.getRepresentingLetter()!="tau"
							&& actParTrans.getRepresentingLetter() != "skip") 
					containedLetters.add(actParTrans.getRepresentingLetter());
			}
			HashSet<String> myAlphabet = new HashSet<String>();
			myAlphabet.addAll((ArrayList<String>)getAlphabet().clone());
			for (Iterator iter = myAlphabet.iterator(); iter.hasNext();) {
				String element = (String) iter.next();
				/* TODO */
				// Don't know if this works since the transitions list is empty
				if (!containedLetters.contains(element))
					trans.add(new ParTransition(this.parent, element));
			}
		} 		
		return trans;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	/**
	 * Generates the alphabet as a union of the alphabets of the children 
	 * taking account of relabeling.
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet!=null && alphabet.isEmpty())
			/*for (Iterator iter = children.iterator(); iter.hasNext();) {
				SimulationNode child = (SimulationNode) iter.next();
				alphabet.addAll(child.getAlphabet());
			}*/
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child = children.get(i);
				child.generateAlphabet();
				alphabet.addAll(child.getAlphabet());
			}
		alphabet.addAll(alphabetExtension);
		for (int i = 0; i < alphabetToHide.size(); i++) {
			String toHide = alphabetToHide.get(i);
			while (alphabet.contains(toHide)) {
				alphabet.remove(toHide);
			}
		}
		if (!relabeling.isEmpty())
			for (int i = 0; i < alphabet.size(); i++) {
				String curLetter = alphabet.get(i);
				if (relabeling.containsKey(curLetter)){
					alphabet.remove(i);
					alphabet.add(relabeling.get(curLetter));
				}				
			}

	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	@Override
	public void takeTransition() {
		System.err.println("Cannot take transition in SequentialProcess: "+this.toString());
	}

	@Override
	public void notificationOfChildActivity(SimulationNode child) {
		if (child.equals(activeChild)) {
			if (parent != null)
				parent.notificationOfChildActivity(this);
		}
		else System.err.println("Wrong argument (" + child.toString() + ") to notificationOfChildActivity in: " 
				+ this.toString() + "\nActive Child is: " + activeChild.toString());
	}
	
	@Override
	public String printTransition() {
		String toReturn = "";
		if (activeChild!=null){
			for (int i = children.indexOf(activeChild); i < children.size(); i++) {
				SimulationNode child = children.get(i);
				String guard = child.getGuard();
				String name = child.isHidden() ? "tau" : child.getName();
				if (guard!="")
					toReturn = toReturn + (name + " (guard: " + guard + ") ; ");
				else toReturn = toReturn + (name + "; ");

			}
			toReturn = toReturn + ("(line " + activeChild.getLine()+ ")\n");
		}
		else {
			for (int i = 0; i < children.size(); i++) {
				SimulationNode child = children.get(i);
				String guard = child.getGuard();
				String name = child.isHidden() ? "tau" : child.getName();
				if (guard!="")
					toReturn = toReturn + (name + "(guard: " + guard + ") ; ");
				else toReturn = toReturn + (name + "; ");
			}
			toReturn = toReturn + ("(line " + children.get(0).getLine()+ ")\n");
			
		}
		return toReturn;
	}
	
	/**
	 * Method sets the notification parameter for an abort of the 
	 * simulation due to a STOP if the process is the root.
	 */
	@Override
	public void stopCalled() {
		if (parent == null) {
			stopped = true;
			releaseTransitions();
		}
		else super.stopCalled();
	}

	/**
	 * @return true if the root process of the simulation was stopped
	 */
	public boolean isStopped() {
		return stopped;
	}
	
	/**
	 * Method sets the notification parameter for an abort of the 
	 * simulation due to an exception thrown if the process is the root.
	 */
	@Override
	public void exceptionThrown(ThrowNode exc) {
		if (parent == null){
			exceptionAbort = true;
			releaseTransitions();
		} else parent.exceptionThrown(exc);
	}

	/**
	 * @return whether the simulation was aborted by an exception or not.
	 */
	public boolean isExceptionAborted() {
		return exceptionAbort;
	}
	
	/**
	 * Method sets the alphabet extension in case of an extend statement.
	 * @param extend
	 */
	public void extendAlphabet(ArrayList<String> extend){
		alphabetExtension = extend;
	}
	
	/**
	 * Method sets the alphabet to hide in case of a hide statement.
	 * @param hide
	 */
	public void hideAlphabet(ArrayList<String> hide){
		alphabetToHide = hide;
	}
	
	/**
	 * Method creates a relabeling map for process in case of a relabel statement.
	 * @param relabel actions to relabel
	 * @param labelby new labelsfor the actions to relabel
	 */
	public void addRelabeling(ArrayList<String> relabel, ArrayList<String> labelby){
		
		for (int i = 0; i < relabel.size(); i++) {
			relabeling.put(relabel.get(i), labelby.get(i));
		}
	}
	
	/**
	 * Method does the actual relabeling during the collection of the 
	 * parallel transitions
	 * @param toRelabel the ParTransitions to relabel
	 * @return the relabeled ParTransitions
	 */
	private List<ParTransition> parRelabel(List<ParTransition> toRelabel) {
		ArrayList<ParTransition> relabeled = new ArrayList<ParTransition>();
		boolean relabelEmpty = relabeling.isEmpty();
		boolean hideEmpty = alphabetToHide.isEmpty();
		if (!relabelEmpty || !hideEmpty){
			for (Iterator iter = toRelabel.iterator(); iter.hasNext();) {
				ParTransition actTrans = (ParTransition) iter.next();
				String newLabel = "";
				if (!relabelEmpty && relabeling.containsKey(actTrans.getRepresentingLetter()))
					newLabel = relabeling.get(actTrans.getRepresentingLetter());
				if (!hideEmpty && alphabetToHide.contains(actTrans.getRepresentingLetter()))
					newLabel = "tau";
				if (newLabel == "")
					relabeled.add(actTrans);
				else {
					actTrans.relabelBy(newLabel);
					relabeled.add(actTrans);
				}				
			}
			return relabeled;
		}		
		else return toRelabel;
	}
	
	private List<SimulationNode> relabel(List<SimulationNode> toRelabel) {
		boolean relabelEmpty = relabeling.isEmpty();
		boolean hideEmpty = alphabetToHide.isEmpty();
		if (!relabelEmpty || !hideEmpty){
			for (Iterator iter = toRelabel.iterator(); iter.hasNext();) {
				SimulationNode actTrans = (SimulationNode)iter.next();
				if (!relabelEmpty && relabeling.containsKey(actTrans.getName()))
					actTrans.relabelNode(relabeling.get(actTrans.getName()));
				if (!hideEmpty && alphabetToHide.contains(actTrans.getName()))
					actTrans.relabelNode("tau");
			}
		}
		return toRelabel;
	}
	
	public void setHighlightInfos(String processName, int line, int column){
		length = processName.length();
		this.line = line;
		this.column = column;
	}
	
	@Override
	public ArrayList<IMarker> createSimHighlighting(IResource resource, IDocument document, boolean isPalt) {
		if (length == 0 && parent !=  null)
			return parent.createSimHighlighting(resource, document, isPalt);
		else 
			return super.createSimHighlighting(resource, document, isPalt);
	}
}
