/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;

import java.util.ArrayList;
import java.util.List;

/**
 * This is a generic node in the stepsimulation tree.
 *
 */
public abstract class SimulationNode {
	
	//* Parent of the node in the tree
	protected SimulationNode parent;
	//* Flag for marking if the node is active in the current state
	protected boolean isActive;
	//* Alphabet of the node
	protected ArrayList<String> alphabet;
	//* Children to the node
	protected ArrayList<SimulationNode> children;
	//* Line of the construct at the root of the node
	protected int line;
	//* Column of the construct at the root of the node
	protected int column;
	//* Length of the construct
	protected int length;
	//* Guard of the node
	protected String guard;
	//* Tells if the action is hidden
	protected boolean hidden;
	//* New label by a relabeling
	protected String newLabel;
	
	
	public SimulationNode() {
		alphabet = new ArrayList<String>();
		children = new ArrayList<SimulationNode>();
		isActive=false;
		guard = "";
		hidden = false;
		newLabel = "";
	}
	
	/**
	 * Constructor setting the line and column of the node. 
	 * @param line
	 * @param column
	 */
	public SimulationNode(int line, int column) {
		this();
		this.line = line;
		this.column = column;
	}
	
	/**
	 * Abstract method for taking a Transition, the node will care for itself how it 
	 * organizes the transition.
	 * @param node
	 */
	public abstract void takeTransition();
	
	/**
	 * Abstract method to generate the alphabet of the node. 
	 */
	public abstract void generateAlphabet();
	
	/**
	 * Method called when the node is completely processed
	 */
	public void isFinished(){
		isActive=false;
	}

	/**
	 * Abstract method for generating the list of possible transitions.
	 * @return the list of transitions
	 */
	public abstract List<SimulationNode> collectTransitions();
	
	/**
	 * Abstract method for generating the list of parallel
	 * transitions of the node. The method also adds disabled 
	 * transitions for the letters in the alphabet that cannot 
	 * be taken.
	 * @return the list of parallel transitions
	 */
	public abstract List<ParTransition> collectParTransitions();
	
	/**
	 * Method which adds a child to the children list
	 * @param child
	 */
	public void addChild(SimulationNode child) {
		children.add(child);
	}
	
	/**
	 * Child notifies the parent that is is finished. Parent has to check if it is finished too.
	 * @param child
	 */
	public abstract void childIsFinished(SimulationNode child);
	
	/**
	 * Method that propagates the break action until it reaches a do. 
	 * Is overwritten in the DoNode class.
	 */
	public void breakTransitionTaken(){
		if (parent!=null) parent.breakTransitionTaken();
	}
	
	/**
	 * Method shall release the active transitions of the node an it's children
	 * in case a do loop is broken or a STOP occured ...
	 */
	public abstract void releaseTransitions();
	
	/**
	 * Method which is called from the child in order to notify the parent that it is active.
	 * @param child
	 */
	public abstract void notificationOfChildActivity(SimulationNode child);
	
	/**
	 * Method that prints the SimulationNode
	 * TODO better incoporation in Eclipse
	 */
	public String printTransition(){
		return ("Error in printTrans of SimulationNode: " + this.toString());
		
	}
	
	/**
	 * Method calling repeatedly the same method in the parent until it reaches the root.
	 * This happens when a STOP action is taken and thus stopps the simulation.
	 */
	public void stopCalled(){
		if (parent != null)
			parent.stopCalled();
	}
	
	/**
	 * Method calling repeatedly the same method in the parent until it reaches a 
	 * TryNode or the root.
	 * @param exc
	 */
	public void exceptionThrown(ThrowNode exc) {
		if (parent != null) parent.exceptionThrown(exc);
	}
	
	public ArrayList<SimulationNode> getParentList(){
		if (parent == null)
			return new ArrayList<SimulationNode>();
		else {
			ArrayList<SimulationNode> parentList = parent.getParentList();
			parentList.add(this);
			return parentList;
		}
	}
	
	/**
	 * Method that creates the markers that do the highlighting in the step
	 * simulation. First creates the markers for itself and then for the parent
	 * hierarchy.
	 * @param resource where to add the markers
	 * @param document needed in order to calculate the offsets
	 * @return a list of markers so that those can be disposed of when not needed anymore
	 */
	public ArrayList<IMarker> createSimHighlighting(IResource resource, IDocument document, boolean isPalt) {
		ArrayList<IMarker> markers = new ArrayList<IMarker>();

		if (line != 0 && column != 0){
			try {
				int line = getLine()-1;
				int column = getColumn();
				int offset = document.getLineOffset(line) + column -1;

				IMarker marker = resource.createMarker(IMarker.PROBLEM);//("modesteditor.core.simulationmarker");//IMarker.TEXT
				marker.setAttribute(IMarker.MESSAGE, "active part of the simulation");
				marker.setAttribute(IMarker.CHAR_START, offset);
				marker.setAttribute(IMarker.CHAR_END, offset+length);
				//marker.setAttribute("isParallel", false);

				markers.add(marker);
			} catch (Exception e){
				System.err.println("Could not create marker in " + this.toString());
			} 
		}
		
		if (parent != null)
			markers.addAll(parent.createSimHighlighting(resource, document, isPalt));
		
		return markers;
	}
	
	public void relabelNode(String newLabel){
		this.newLabel = newLabel;
	}

	public boolean isActive() {
		return isActive;
	}

	public void setActive() {
		this.isActive = true;
		//System.err.println("vor generateAlphabet");
		generateAlphabet();
		//System.err.println("nach simulationNode setActive()");
	}

	public SimulationNode getParent() {
		return parent;
	}

	public void setParent(SimulationNode parent) {
		this.parent = parent;
	}

	public ArrayList<String> getAlphabet() {
		return alphabet;
	}
	
	public int getColumn() {
		return column;
	}

	public int getLength() {
		return length;
	}

	public int getLine() {
		return line;
	}
	
	public boolean isHidden() {
		return hidden;
	}
	
	/**
	 * Method hides the action for printing.
	 */
	public void hide(){
		hidden = true;
	}

	public String getName() {
		return "";
	}

	public String getGuard() {
		return guard;
	}
}
