/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import modesteditor.core.ModestEditor;

import java.util.ArrayList;

/**
 * Transition class between the simulatin model 
 * and the simulation framework like the view and the action.
 */
public class SimulationRoot {

	private ArrayList<SimulationNode> transitions;
	private ArrayList<PaltTransition> pAlternatives;
	private ArrayList<ISimulationListener> listeners;
	
	private SequentialProcess root;
	
	private boolean isFinished;
	private String finishingMessage;
	
	
	
	public SimulationRoot(SequentialProcess proc) {
		root=proc;
		transitions = new ArrayList<SimulationNode>();
		pAlternatives = new ArrayList<PaltTransition>();
		listeners = new ArrayList<ISimulationListener>();
		isFinished = false;
		finishingMessage = "";
		
	}
	
	/**
	 * Method calls update and delivers the possible 
	 * transitions to the content provider.
	 * @return the possible transitions
	 */
	public Object[] getModels() {
		//if (transitions.isEmpty())
		
		if (pAlternatives.isEmpty()){
			update();

			//System.err.println("SimulationRoot: size of transitions is " + transitions.size());
			/*for (int i = 0; i < transitions.size(); i++) {
				String trans = transitions.get(i).printTransition();
				System.err.println("Trans: " + trans);
			}*/
			
			return transitions.toArray();
		} else 
			return pAlternatives.toArray();
	}
	
	/**
	 * Method updates the transitions and also the finishing message if necessary.
	 *
	 */
	private void update(){
		transitions = (ArrayList<SimulationNode>)root.collectTransitions();
		if (transitions.isEmpty()) { //System.err.println("Cannot do anything anymore");
			isFinished=true;
			finishingMessage = "No transitions to take, step simulation aborting";
		}
			
		if (!root.isActive())  {
			isFinished=true;
			if (root.isStopped())
				finishingMessage = "Simulation stopped by a STOP action";
			else if (root.isExceptionAborted())
				finishingMessage = "Simulation stopped by a a thrown exception";
			else finishingMessage = "Simulation exited normally";
		} 
	}
	
	public void addListener(ISimulationListener list) {
		listeners.add(list);
	}
	
	public void removeListener(ISimulationListener list) {
		listeners.remove(list);
	}
	
	public void removeEditorListeners() {
		for (int i = 0; i < listeners.size(); i++) {
			if (listeners.get(i) instanceof ModestEditor)
				listeners.remove(i);
		}
	}
	
	/**
	 * Notifies all listeners of a change in the simulation.
	 *
	 */
	private void fireModelChanged() {
		for (int i = 0; i < listeners.size(); i++) {
			listeners.get(i).simulationChanged(this);
		}
		
	}
	
	/**
	 * Method called when a transition is chosen in the SimulationView.
	 * @param alt
	 * @param isInBackTracking tells if the choice is taken during backtracking
	 */
	public void choiceTaken(SimulationNode alt, boolean isInBackTracking){
		if (isInBackTracking){
			if (alt instanceof PaltTransition){
				alt.takeTransition();
				pAlternatives.clear();

			} else {
				if (alt instanceof ParTransition)
					pAlternatives = ((ParTransition)alt).getPaltTransitions();
				if (alt instanceof PaltNode)
					pAlternatives = ((PaltNode)alt).getPaltTransitions();
				alt.takeTransition();

			}
		} else {
			if (alt instanceof PaltTransition){
				alt.takeTransition();
				pAlternatives.clear();
				fireModelChanged();
			} else {
				if (alt instanceof ParTransition)
					pAlternatives = ((ParTransition)alt).getPaltTransitions();
				if (alt instanceof PaltNode)
					pAlternatives = ((PaltNode)alt).getPaltTransitions();
				alt.takeTransition();
				
				// Check if only one P-Alternative is available
				if (pAlternatives.size() == 1) {
					pAlternatives.get(0).takeTransition();
					pAlternatives.clear();
				}
				fireModelChanged();
			}
		}
	}

	/**
	 * @return the finishingMessage
	 */
	public String getFinishingMessage() {
		return finishingMessage;
	}

	/**
	 * @return if the simulation is finished
	 */
	public boolean isFinished() {
		return isFinished;
	}
	
	/**
	 * Clears the state in die SimulationRoot in order to reprocess
	 * the transitions for backtracking.
	 *
	 */
	public void clearState() {
		isFinished = false;
		root.releaseTransitions();
		root.setActive();
	}
	
	
}
