/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.List;

/**
 * Calss representing a throw statement in the simulation tree.
 *
 */
public class ThrowNode extends SimulationNode {
	
	private String exceptionName;
	
	private boolean isCaught;

	
	public ThrowNode(int line, int column) {
		super(line, column);
		isCaught = false;
	}
	
	
	public ThrowNode(int line, int column, String exceptionName) {
		this(line, column);
		this.exceptionName = exceptionName;
	}
	
	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param exceptionName the name of the exception which is eventually thrown
	 * @param parent the parent node in the simulation tree
	 * @param guard
	 */
	public ThrowNode(int line, int column, String exceptionName, SimulationNode parent, String guard) {
		this(line, column, exceptionName);
		setParent(parent);
		this.guard = guard;
		if (parent != null) parent.addChild(this);
		length = 5;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {}

	/**
	 * @return a ParTransition representing this thrwo statement.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectParTransitions()
	 */
	@Override
	public List<ParTransition> collectParTransitions() {
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			ParTransition parTrans = new ParTransition(parent, exceptionName);
			parTrans.setThrowStatement();
			parTrans.addTransition(this);
			trans.add(parTrans);
		}
		return trans;
	}

	/**
	 * @return the node itself as a transition.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()){ 
			ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
			trans.add(this);
			return ((List<SimulationNode>) trans);			
		} else return null;
	}

	/**
	 * Method does nothing since the throw statement has no alphabet.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#notificationOfChildActivity(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void notificationOfChildActivity(SimulationNode child) {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#releaseTransitions()
	 */
	@Override
	public void releaseTransitions() {
		isActive=false;
	}

	/**
	 * If the transition of this throw statement is taken, 
	 * the exceptionThrown method is called which passes the 
	 * exception up in the tree until a catch statement is reached.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition()
	 */
	@Override
	public void takeTransition() {
		exceptionThrown(this);
		isFinished(); 
	}

	/**
	 * @return the exceptionName
	 */
	public String getExceptionName() {
		return exceptionName;
	}

	@Override
	public String printTransition() {
		String label = isCaught ? "tau" : exceptionName;
		return "["+guard+"] " + label ;//"; (line " + getLine() + ")\n";
	}
	
	/**
	 * Method that signals that the thrown exception will be 
	 * caught later on.
	 */
	public void willBeCaught() {
		isCaught = true;
	}
	
}
