/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing a try block with it's catch statements.
 *
 */
public class TryNode extends SimulationNode {

//	* The active child, for a start the process in the body, later maybe a catch statement
	private SimulationNode activeChild;
	
	private HashMap<String, SimulationNode> catchMap;
	
	private ArrayList<SimulationNode> catches;
	
	public TryNode(int line, int column) {
		super(line, column);
		activeChild = null;
		catchMap = new HashMap<String, SimulationNode>();
		catches = new ArrayList<SimulationNode>();
	}
	
	public TryNode(int line, int column, SimulationNode parent) {
		this(line, column);
		setParent(parent);
		if (parent != null) parent.addChild(this);
		length = 3;
	}
	
	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param parent
	 * @param guard
	 */
	public TryNode(int line, int column, SimulationNode parent, String guard) {
		this(line, column, parent);
		this.guard = guard;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (child != activeChild) 
			System.err.println("Wrong argument to childIsFinished for TryNode: " + this.toString());
		else 
			isFinished();
	}
	
	@Override
	public void isFinished() {
		super.isFinished();
		activeChild = null;
		parent.childIsFinished(this);
		
	}

	/**
	 * Method collects the ParTransitions of the active child.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectParTransitions()
	 */
	@Override
	public List<ParTransition> collectParTransitions() {
		List<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			trans = activeChild.collectParTransitions();
			HashSet<String> containedLetters = new HashSet<String>();
			for (int i = 0; i < trans.size(); i++) {
				ParTransition actParTrans = trans.get(i);
				if (actParTrans.getRepresentingLetter()!="break"
					&& actParTrans.getRepresentingLetter()!="stop"
						&& actParTrans.getRepresentingLetter()!="tau"
							&& actParTrans.getRepresentingLetter()!="skip") 
					containedLetters.add(actParTrans.getRepresentingLetter());
				if (actParTrans.isThrowStatement()){
					if (catchMap.containsKey(actParTrans.getRepresentingLetter()))
						actParTrans.willBeCaught();
				}
			}
			HashSet<String> myAlphabet = new HashSet<String>();
			myAlphabet.addAll((ArrayList<String>)getAlphabet().clone());
			for (Iterator iter = myAlphabet.iterator(); iter.hasNext();) {
				String element = (String) iter.next();
				/* TODO */
				// Don't know if this works since the transitions list is empty
				if (!containedLetters.contains(element))
					trans.add(new ParTransition(this.parent, element));
			}
		} 		
		return trans;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()) {
			ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>(activeChild.collectTransitions());
			for (int i = 0; i < trans.size(); i++) {
				SimulationNode possibleThrow = trans.get(i);
				if (possibleThrow instanceof ThrowNode) {
					ThrowNode actualThrow = (ThrowNode) possibleThrow;
					if (catchMap.containsKey(actualThrow.getExceptionName()))
						actualThrow.willBeCaught();
				}
			}
			return trans;
		} else
			return new ArrayList<SimulationNode>();
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet != null && alphabet.isEmpty()){
			if (activeChild != null) 
				alphabet.addAll(activeChild.getAlphabet());
			for (int i = 0; i < catches.size(); i++) {
				SimulationNode catchi = catches.get(i);
				catchi.generateAlphabet();
				alphabet.addAll(catchi.getAlphabet());
			}
		}
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#notificationOfChildActivity(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void notificationOfChildActivity(SimulationNode child) {
		// Do I need something here? Maybe notify the parent ;)
		parent.notificationOfChildActivity(this);
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#releaseTransitions()
	 */
	@Override
	public void releaseTransitions() {
		if (isActive()){
			isActive = false;
			activeChild.releaseTransitions();
			activeChild = null;
		}
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition()
	 */
	@Override
	public void takeTransition() {
		System.err.println("Cannot take transition in TryNode: "+this.toString());
	}
	
	@Override
	public void setActive() {
		
		if (activeChild == null && !children.isEmpty()){
			// Should have only one seq_process as child
			activeChild = children.get(0);
			activeChild.setActive();
		}
		super.setActive();
	}

	public void addCatch(SimulationNode ca, String exc) {
		catches.add(ca);
		catchMap.put(exc, ca);
		ca.setParent(this);
		
	}
	
	@Override
	public void exceptionThrown(ThrowNode exc) {
		String exception = exc.getExceptionName();
		
		if (catchMap.containsKey(exception)){
			// Relax all transitions and reset the child process in the try-block
			activeChild.releaseTransitions();
			activeChild = catchMap.get(exception);
			activeChild.setActive();
			
		}	
		else parent.exceptionThrown(exc);
	}
}
