/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */

package modesteditor.launch;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.debug.core.model.IProcess;


/**
 * This class performs the actual task of running the compiler 
 * and getting the C++ soures out of it or the dot file.
 */
public class ModestLaunchDelegate implements ILaunchConfigurationDelegate {

	/** Performs a launch. This method is called by Eclipse when the user
	 * wants to run something.
	 * <p>
	 * The information entered by the user in the 'Run...' dialog
	 * is passed here in the given launch configuration. Eclipse also
	 * creates a new launch object to manage the processes we will create.
	 * This method's job is only to run the modest compiler in a new 
	 * process and add it to the given launch object.
	 */
	public void launch (ILaunchConfiguration config, String mode, ILaunch launch, IProgressMonitor monitor) throws CoreException {
		IPath filePath = getFilePath(config);
		if (filePath == null)
			throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "The file to launch was not found in the workspace.", null));
		
		IPath fileDir = filePath.removeLastSegments(1);
		
		//Choice whether to compile the file or to run fsns
		if (!config.getAttribute(ModestLaunchPlugin.FSNS_RUN, false))
		{
			boolean dotFileCreation = (ModestLaunchPlugin.getDefault().getPreferenceStore().getBoolean(ModestLaunchPlugin.DOT_FILE_CREATION)
				|| config.getAttribute(ModestLaunchPlugin.DOT_FILE_CREATION, false));
		
		File workingDir;
		String outputDirectory = ModestLaunchPlugin.getDefault().getPreferenceStore().getString(ModestLaunchPlugin.OUTPUT_FOLDER_LOCATION);
		
		if (outputDirectory == "") workingDir = fileDir.toFile();
		else workingDir = new File(outputDirectory);
			
		
		IProcess modestProcess = execModest(config, workingDir, launch, dotFileCreation);
		
		
		// TODO Warteschleife, evtl. anders lsen(Threads, waits?)
		do {
			//System.err.println("Am warten");
		} while (!modestProcess.isTerminated());
		//System.err.println("Fertig mit warten");
		
		
		
		// TODO get the commons library to load with the start of the plugin
		ArrayList<String> cpFiles = getCopyCmdLine(config);
		//for (Iterator iter = cpFiles.iterator(); iter.hasNext();) {
		for (int i = 0; i < cpFiles.size(); i++) {
		
			try {
				String filename = (String) cpFiles.get(i);
				File actFile = new File(filename);
				if (!dotFileCreation) FileUtils.copyFileToDirectory(actFile, workingDir);
				FileUtils.forceDelete(actFile);
			} catch (IOException e) {
				throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "The files to be copied after compile are not there.", null));
			}
		}

		} 
		
		//or do the fsns run.
		else {
			IProcess fsnsProcess = execFSNS(config, fileDir.toFile(), launch);
		}
		
		
	}
	
	/** Runs the modest compiler in a new process with the given
	 * working directory, and add it to the given launch object. */
	private IProcess execModest (ILaunchConfiguration config, File workingDir, ILaunch launch, boolean dotFileCreation) throws CoreException {
		ArrayList<String> cmdLine = new ArrayList<String>();
		String momodestPath = ModestLaunchPlugin.getDefault().getPreferenceStore().getString(ModestLaunchPlugin.COMPILER_LOCATION);
		
		if (momodestPath == null || momodestPath.length() == 0)
			throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "No momodest binary specified in the launch preferences.", null));
		
		// Steps which build the actual commandline by adding successively the arguments, 
		// such as executable, options and file to compile
		cmdLine.add(momodestPath);
		
		if (dotFileCreation)
			cmdLine.add("--dot");
		
		IPath filePath = getFilePath(config);
		if (filePath == null)
			throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "The file to launch was not found in the workspace.", null));
		cmdLine.add(filePath.toOSString());
		
		/*if (cpCmdLine != null && !dotFileCreation)
		{
			//cmdLine.add("; ");
			for (Iterator iter = cpCmdLine.iterator(); iter.hasNext();) 
				cmdLine.add((String) iter.next());
		}*/
		
		// Debug output to see what the actual command line looks like.
		//for (int i=0; i<cmdLine.size(); i++) System.err.print(cmdLine.get(i)+ " ");
		//System.err.println("\nWorking directory: " + workingDir.toString());
					
		Process process = DebugPlugin.exec(cmdLine.toArray(new String[0]), workingDir);
		return DebugPlugin.newProcess(launch, process, "MoDeST");
	}
	
	/** Runs the modest fsns tool in a new process with the given
	 * working directory, and add it to the given launch object. */
	private IProcess execFSNS (ILaunchConfiguration config, File workingDir, ILaunch launch) throws CoreException {
		ArrayList<String> cmdLine = new ArrayList<String>();
		String fsnsPath = ModestLaunchPlugin.getDefault().getPreferenceStore().getString(ModestLaunchPlugin.FSNS_LOCATION);
		
		if (fsnsPath == null || fsnsPath.length() == 0)
			throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "No fsns binary specified in the launch preferences.", null));
		
		// Steps which build the actual commandline by adding successively the arguments, 
		// such as executable, options and file to compile
		cmdLine.add(fsnsPath);
		cmdLine.add("--quiet");
				
		IPath filePath = getFilePath(config);
		if (filePath == null)
			throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "The file to launch was not found in the workspace.", null));
		cmdLine.add(filePath.toOSString());
		
		/*if (cpCmdLine != null && !dotFileCreation)
		{
			//cmdLine.add("; ");
			for (Iterator iter = cpCmdLine.iterator(); iter.hasNext();) 
				cmdLine.add((String) iter.next());
		}*/
		
		// Debug output to see what the actual command line looks like.
		//for (int i=0; i<cmdLine.size(); i++) System.err.print(cmdLine.get(i)+ " ");
		//System.err.println("\nWorking directory: " + workingDir.toString());
					
		Process process = DebugPlugin.exec(cmdLine.toArray(new String[0]), workingDir);
		return DebugPlugin.newProcess(launch, process, "MoDeST: fsns");
	}

	/** Queries the given launch configuration for the file to be run,
	 * and returns its absolute path. */
	private IPath getFilePath (ILaunchConfiguration config) throws CoreException {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		String projectName = config.getAttribute(ModestLaunchPlugin.MODEST_CONFIG_PROJECT, "");
		String fileName = config.getAttribute(ModestLaunchPlugin.MODEST_COMPILE_FILE, "");
		IProject project = root.getProject(projectName);
		IFile file = project.getFile(fileName);
		if (file.exists()) 
			return file.getLocation();
		else
			return null;
	}
	
	private ArrayList<String> getCopyCmdLine(ILaunchConfiguration config) throws CoreException{
		ArrayList<String> cpCmdLine = new ArrayList<String>();
		
		IPath filePath = getFilePath(config);
		if (filePath == null)
			throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "The file to launch was not found in the workspace.", null));
		IPath fileDir = filePath.removeLastSegments(1);
		String fileName = filePath.lastSegment();
		
		if (fileName.endsWith(".modest")) {
			String baseName = fileName.substring(0, fileName.length()-7);
			String folder = fileDir.toOSString();
			cpCmdLine.add(folder+"/"+baseName+"_dependencies.cpp");
			cpCmdLine.add(folder+"/"+baseName+"EXT.cpp");
			cpCmdLine.add(folder+"/"+baseName+"EXT.exd");
			cpCmdLine.add(folder+"/"+baseName+"EXT.h");
			cpCmdLine.add(folder+"/"+baseName+"_init.cpp");
			cpCmdLine.add(folder+"/"+baseName+".ttd");
		 
		} else throw new CoreException(new Status(IStatus.ERROR, "modesteditor.launch", 1, "The file to launch doesn't end with '.modest'.", null));
		
		return cpCmdLine;
	}
	

}
