/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.launch.ui;

import modesteditor.launch.ModestLaunchPlugin;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * The main tab in the Run dialog for MoDeST. Allows the user to enter
 * the file to be compiled, and saves the information in a launch configuration.
 * <p>
 * Note that we never have to create our own launch configurations
 * ("configs"). Eclipse handles the entire config life cycle --
 * creating, saving and reloading configs -- on its own.
 * This class is only responsible for getting MoDeST-specific information
 * from the user and entering it in the config Eclipse passes to it.
 * Eclipse will save the config somewhere, and pass it to the
 * MoDeST launch delegate whenever the user asks to run the config.
 * To enable Eclipse to do all this, a launch configuration is very simple,
 * essentially just a mapping from strings to a few simple types of data.
 */
public class ModestLaunchTab extends AbstractLaunchConfigurationTab {

	private Text projectField, fileField;
	private Button fsnsCheckBox;
	private Button dotCheckBox;
	private ModifyListener modifyListener = new ModifyListener () {
		public void modifyText (ModifyEvent e) {
			updateLaunchConfigurationDialog();
		}
	};
	
	private SelectionListener selectionListener = new SelectionListener () {
		public void widgetSelected(SelectionEvent e){
			updateLaunchConfigurationDialog();
		}
		
		public void widgetDefaultSelected(SelectionEvent e){
			updateLaunchConfigurationDialog();
		}
	};
	private Image image;
	
	
	/** Creates the contents of this tab. The given composite is the container
	 * which is to be populated with the tab's fields. */
	/* TODO: The fields should use Resource- or Element- ListSelectionDialog */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		setControl(composite);
		composite.setLayout(new GridLayout());
		
		new Label(composite, SWT.NONE).setText("Project:");
		projectField = new Text(composite, SWT.SINGLE | SWT.BORDER);
		projectField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		projectField.addModifyListener(modifyListener);
		
		Group fileGroup = new Group(composite, SWT.NULL);
		fileGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fileGroup.setLayout(new GridLayout(2,false));
		fileGroup.setText("File");
		
		fsnsCheckBox = new Button(fileGroup,SWT.CHECK);
		fsnsCheckBox.setText("run fsns");
		fsnsCheckBox.addSelectionListener(selectionListener);
		
		dotCheckBox = new Button(fileGroup,SWT.CHECK);
		dotCheckBox.setText("dot file generation");	
		dotCheckBox.addSelectionListener(selectionListener);
		
		fileField = new Text(fileGroup, SWT.SINGLE | SWT.BORDER);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		fileField.setLayoutData(gridData);
		fileField.addModifyListener(modifyListener);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
	 */
	public String getName() {
		return "Main";
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
	 */
	public void initializeFrom(ILaunchConfiguration configuration) {
		try {
			projectField.setText(configuration.getAttribute(ModestLaunchPlugin.MODEST_CONFIG_PROJECT, ""));
			fileField.setText(configuration.getAttribute(ModestLaunchPlugin.MODEST_COMPILE_FILE, ""));
			fsnsCheckBox.setSelection(configuration.getAttribute(ModestLaunchPlugin.FSNS_RUN, false));
			dotCheckBox.setSelection(configuration.getAttribute(ModestLaunchPlugin.DOT_FILE_CREATION, false));
		} catch (CoreException e) {
			projectField.setText("");
			fileField.setText("");
			fsnsCheckBox.setSelection(false);
			dotCheckBox.setSelection(false);
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		configuration.setAttribute(ModestLaunchPlugin.MODEST_CONFIG_PROJECT, projectField.getText());
		configuration.setAttribute(ModestLaunchPlugin.MODEST_COMPILE_FILE, fileField.getText());
		configuration.setAttribute(ModestLaunchPlugin.FSNS_RUN, fsnsCheckBox.getSelection());
		configuration.setAttribute(ModestLaunchPlugin.DOT_FILE_CREATION, dotCheckBox.getSelection());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
		// leave the config empty

	}
	
	public Image getImage () {
		if (image == null) {
			String url = "/icons/file.gif";
			ImageDescriptor descriptor = ImageDescriptor.createFromURL(ModestLaunchPlugin.getDefault().getBundle().getEntry(url));
			image = descriptor.createImage();
		}
		return image;
	}
	
	/** Called when the tab has been disposed.
	 * Used to perform any cleanup required. */
	public void dispose () {
		if (image != null)
			image.dispose();
		super.dispose();
	}

}
